'''OpenGL extension SUN.mesh_array

This module customises the behaviour of the 
OpenGL.raw.GL.SUN.mesh_array to provide a more 
Python-friendly API

Overview (from the spec)
	
	
	This extension defines a new mesh primitive.
	The primitive can only be used with vertex arrays and cannot be used in
	immediate mode. The application must arrange the vertices in row major format.
	For example if a quad mesh is 4 vertices wide the, vertices in the first
	row are the first 4 vertices and vertices in the second row are vertices 5
	through 8. 

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SUN/mesh_array.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.SUN.mesh_array import *
from OpenGL.raw.GL.SUN.mesh_array import _EXTENSION_NAME

def glInitMeshArraySUN():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ARB.pixel_buffer_object

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.pixel_buffer_object to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension expands on the interface provided by the
	ARB_vertex_buffer_object extension (and later integrated into OpenGL
	1.5) in order to permit buffer objects to be used not only with vertex
	array data, but also with pixel data.  The intent is to provide more
	acceleration opportunities for OpenGL pixel commands.
	
	While a single buffer object can be bound for both vertex arrays and
	pixel commands, we use the designations vertex buffer object (VBO)
	and pixel buffer object (PBO) to indicate their particular usage in
	a given situation.
	
	Recall that buffer objects conceptually are nothing more than arrays
	of bytes, just like any chunk of memory.  ARB_vertex_buffer_object
	allows GL commands to source data from a buffer object by binding the
	buffer object to a given target and then overloading a certain set of
	GL commands' pointer arguments to refer to offsets inside the buffer,
	rather than pointers to user memory.  An offset is encoded in a
	pointer by adding the offset to a null pointer.
	
	This extension does not add any new functionality to buffer objects
	themselves.  It simply adds two new targets to which buffer objects
	can be bound: GL_PIXEL_PACK_BUFFER and GL_PIXEL_UNPACK_BUFFER.  When a
	buffer object is bound to the GL_PIXEL_PACK_BUFFER target, commands
	such as glReadPixels pack (write) their data into a buffer object.
	When a buffer object is bound to the GL_PIXEL_UNPACK_BUFFER target,
	commands such as glDrawPixels and glTexImage2D unpack (read) their
	data from a buffer object.
	
	There are a several approaches to improve graphics performance
	with PBOs.  Some of the most interesting approaches are:
	
	- Streaming texture updates:  If the application uses
	  glMapBuffer/glUnmapBuffer to write its data for glTexSubImage into
	  a buffer object, at least one of the data copies usually required
	  to download a texture can be eliminated, significantly increasing
	  texture download performance.
	
	- Streaming draw pixels: When glDrawPixels sources client memory,
	  OpenGL says the client memory can be modified immediately after the
	  glDrawPixels command returns without disturbing the drawn image.
	  This typically necessitates unpacking and copying the image prior
	  to glDrawPixels returning.  However, when using glDrawPixels with
	  a pixel pack buffer object, glDrawPixels may return prior to image
	  unpacking because future modification of the buffer data requires
	  explicit commands (glMapBuffer, glBufferData, or glBufferSubData).
	
	- Asynchronous glReadPixels:  If an application needs to read back a
	  number of images and process them with the CPU, the existing GL
	  interface makes it nearly impossible to pipeline this operation.
	  The driver will typically send the hardware a readback command
	  when glReadPixels is called, and then wait for all of the data to
	  be available before returning control to the application.  Then,
	  the application can either process the data immediately or call
	  glReadPixels again; in neither case will the readback overlap with
	  the processing.  If the application issues several readbacks
	  into several buffer objects, however, and then maps each one to
	  process its data, then the readbacks can proceed in parallel with
	  the data processing.
	
	- Render to vertex array:  The application can use a fragment
	  program to render some image into one of its buffers, then read
	  this image out into a buffer object via glReadPixels.  Then, it can
	  use this buffer object as a source of vertex data.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/pixel_buffer_object.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.pixel_buffer_object import *
from OpenGL.raw.GL.ARB.pixel_buffer_object import _EXTENSION_NAME

def glInitPixelBufferObjectARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension AMD.multi_draw_indirect

This module customises the behaviour of the 
OpenGL.raw.GL.AMD.multi_draw_indirect to provide a more 
Python-friendly API

Overview (from the spec)
	
	The ARB_draw_indirect extension (included in OpenGL 4.0) introduced
	mechanisms whereby the parameters for a draw function may be provided in
	a structure contained in a buffer object rather than as parameters to the
	drawing procedure. This is known as an indirect draw and is exposed as two
	new functions, glDrawArraysIndirect and glDrawElementsIndirect. Each of
	these functions generates a single batch of primitives.
	
	This extension builds on this functionality by providing procedures to
	invoke multiple draws from a single procedure call. This allows large
	batches of drawing commands to be assembled in server memory (via a buffer
	object) which may then be dispatched through a single function call.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/AMD/multi_draw_indirect.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.AMD.multi_draw_indirect import *
from OpenGL.raw.GL.AMD.multi_draw_indirect import _EXTENSION_NAME

def glInitMultiDrawIndirectAMD():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_depth_buffer_float'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_depth_buffer_float',error_checker=_errors._error_checker)
GL_DEPTH32F_STENCIL8=_C('GL_DEPTH32F_STENCIL8',0x8CAD)
GL_DEPTH_COMPONENT32F=_C('GL_DEPTH_COMPONENT32F',0x8CAC)
GL_FLOAT_32_UNSIGNED_INT_24_8_REV=_C('GL_FLOAT_32_UNSIGNED_INT_24_8_REV',0x8DAD)




"""FreeGLUT extensions to the GLUT API

This module will provide the FreeGLUT extensions if they are available
from the GLUT module.  Note that any other implementation that also provides
these entry points will also retrieve the entry points with this module.
"""
# flags 'freeglut_ext.xml -l /usr/lib64/libglut.so -o freeglut_ext.py -v -kf'
from OpenGL import platform, arrays
from OpenGL import constant
FUNCTION_TYPE = platform.PLATFORM.functionTypeFor( platform.PLATFORM.GLUT )
from OpenGL.GLUT import special
from OpenGL import wrapper as _wrapper
from OpenGL.raw.GL._types import *

import ctypes
c_int = ctypes.c_int 
c_char_p = ctypes.c_char_p
c_ubyte = ctypes.c_ubyte
c_void_p = ctypes.c_void_p

GLUT_DEBUG = constant.Constant( 'GLUT_DEBUG', 0x0001 )
GLUT_FORWARD_COMPATIBLE = constant.Constant( 'GLUT_FORWARD_COMPATIBLE',  0x0002)

GLUT_ACTION_EXIT = constant.Constant( 'GLUT_ACTION_EXIT', 0 )
GLUT_ACTION_GLUTMAINLOOP_RETURNS = constant.Constant( 'GLUT_ACTION_GLUTMAINLOOP_RETURNS', 1 )
GLUT_ACTION_CONTINUE_EXECUTION = constant.Constant( 'GLUT_ACTION_CONTINUE_EXECUTION', 2 )

GLUT_INIT_MAJOR_VERSION = constant.Constant( 'GLUT_INIT_MAJOR_VERSION', 0x0200 )
GLUT_INIT_MINOR_VERSION = constant.Constant( 'GLUT_INIT_MINOR_VERSION', 0x0201 )
GLUT_INIT_FLAGS = constant.Constant( 'GLUT_INIT_FLAGS', 0x0202 )

GLUT_CREATE_NEW_CONTEXT = constant.Constant( 'GLUT_CREATE_NEW_CONTEXT', 0 )
GLUT_USE_CURRENT_CONTEXT = constant.Constant( 'GLUT_USE_CURRENT_CONTEXT', 1 )

GLUT_ACTION_ON_WINDOW_CLOSE = constant.Constant( 'GLUT_ACTION_ON_WINDOW_CLOSE', 0x01F9 )
GLUT_WINDOW_BORDER_WIDTH = constant.Constant( 'GLUT_WINDOW_BORDER_WIDTH', 0x01FA )
GLUT_WINDOW_HEADER_HEIGHT = constant.Constant( 'GLUT_USE_CURRENT_CONTEXT', 0x01FB )
#GLUT_VERSION = constant.Constant( 'GLUT_VERSION', 0x01FC )
GLUT_RENDERING_CONTEXT = constant.Constant( 'GLUT_RENDERING_CONTEXT', 0x01FD )

GLUT_ALLOW_DIRECT_CONTEXT=1
GLUT_AUX=0x1000
GLUT_AUX1=0x1000
GLUT_AUX2=0x2000
GLUT_AUX3=0x4000
GLUT_AUX4=0x8000
GLUT_BORDERLESS=0x0800
GLUT_CAPTIONLESS=0x0400
GLUT_COMPATIBILITY_PROFILE=0x0002
GLUT_CORE_PROFILE=0x0001
GLUT_DIRECT_RENDERING=0x01
GLUT_FORCE_DIRECT_CONTEXT=3
GLUT_FORCE_INDIRECT_CONTEXT=0
GLUT_FULL_SCREEN=0x01
GLUT_INIT_PROFILE=0x0203
GLUT_KEY_BEGIN=0x006
GLUT_KEY_DELETE=0x006
GLUT_KEY_NUM_LOCK=0x006
GLUT_SRGB=0x1000
GLUT_TRY_DIRECT_CONTEXT=2

# /usr/include/GL/freeglut_ext.h 63
glutMainLoopEvent = platform.createBaseFunction( 
    'glutMainLoopEvent', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[],
    doc='glutMainLoopEvent(  ) -> None', 
    argNames=(),
)
# /usr/include/GL/freeglut_ext.h 64
glutLeaveMainLoop = platform.createBaseFunction( 
    'glutLeaveMainLoop', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[],
    doc='glutLeaveMainLoop(  ) -> None', 
    argNames=(),
)

# /usr/include/GL/freeglut_ext.h 69
##glutMouseWheelFunc = platform.createBaseFunction( 
##	'glutMouseWheelFunc', dll=platform.PLATFORM.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)],
##	doc='glutMouseWheelFunc( FUNCTION_TYPE(None, c_int, c_int, c_int, c_int)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutMouseWheelFunc = special.GLUTCallback(
    'MouseWheel', (c_int, c_int, c_int, c_int,), ('wheel','direction','x','y'),
)


# /usr/include/GL/freeglut_ext.h 70
##glutCloseFunc = platform.createBaseFunction( 
##	'glutCloseFunc', dll=platform.PLATFORM.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None)],
##	doc='glutCloseFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutCloseFunc = special.GLUTCallback(
    'Close', (), (),
)

# /usr/include/GL/freeglut_ext.h 71
##glutWMCloseFunc = platform.createBaseFunction( 
##	'glutWMCloseFunc', dll=platform.PLATFORM.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None)],
##	doc='glutWMCloseFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutWMCloseFunc = special.GLUTCallback(
    'WMClose', (), (),
)

# /usr/include/GL/freeglut_ext.h 73
##glutMenuDestroyFunc = platform.createBaseFunction( 
##	'glutMenuDestroyFunc', dll=platform.PLATFORM.GLUT, resultType=None, 
##	argTypes=[FUNCTION_TYPE(None)],
##	doc='glutMenuDestroyFunc( FUNCTION_TYPE(None)(callback) ) -> None', 
##	argNames=('callback',),
##)
glutMenuDestroyFunc = special.GLUTCallback(
    'MenuDestroy', (), (),
)

# /usr/include/GL/freeglut_ext.h 78
glutSetOption = platform.createBaseFunction( 
    'glutSetOption', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[GLenum, c_int],
    doc='glutSetOption( GLenum(option_flag), c_int(value) ) -> None', 
    argNames=('option_flag', 'value'),
)

# /usr/include/GL/freeglut_ext.h 80
glutGetWindowData = platform.createBaseFunction( 
    'glutGetWindowData', dll=platform.PLATFORM.GLUT, resultType=c_void_p, 
    argTypes=[],
    doc='glutGetWindowData(  ) -> c_void_p', 
    argNames=(),
)

# /usr/include/GL/freeglut_ext.h 81
glutSetWindowData = platform.createBaseFunction( 
    'glutSetWindowData', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[c_void_p],
    doc='glutSetWindowData( c_void_p(data) ) -> None', 
    argNames=('data',),
)

# /usr/include/GL/freeglut_ext.h 82
glutGetMenuData = platform.createBaseFunction( 
    'glutGetMenuData', dll=platform.PLATFORM.GLUT, resultType=c_void_p, 
    argTypes=[],
    doc='glutGetMenuData(  ) -> c_void_p', 
    argNames=(),
)

# /usr/include/GL/freeglut_ext.h 83
glutSetMenuData = platform.createBaseFunction( 
    'glutSetMenuData', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[c_void_p],
    doc='glutSetMenuData( c_void_p(data) ) -> None', 
    argNames=('data',),
)

# /usr/include/GL/freeglut_ext.h 88
glutBitmapHeight = platform.createBaseFunction( 
    'glutBitmapHeight', dll=platform.PLATFORM.GLUT, resultType=c_int, 
    argTypes=[c_void_p],
    doc='glutBitmapHeight( c_void_p(font) ) -> c_int', 
    argNames=('font',),
)

# /usr/include/GL/freeglut_ext.h 89
glutStrokeHeight = platform.createBaseFunction( 
    'glutStrokeHeight', dll=platform.PLATFORM.GLUT, resultType=GLfloat, 
    argTypes=[c_void_p],
    doc='glutStrokeHeight( c_void_p(font) ) -> GLfloat', 
    argNames=('font',),
)

# /usr/include/GL/freeglut_ext.h 90
glutBitmapString = platform.createBaseFunction( 
    'glutBitmapString', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[c_void_p, c_char_p],
    doc='glutBitmapString( c_void_p(font), POINTER(c_ubyte)(string) ) -> None', 
    argNames=('font', 'string'),
)

# /usr/include/GL/freeglut_ext.h 91
glutStrokeString = platform.createBaseFunction( 
    'glutStrokeString', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[c_void_p, c_char_p],
    doc='glutStrokeString( c_void_p(font), POINTER(c_ubyte)(string) ) -> None', 
    argNames=('font', 'string'),
)

# /usr/include/GL/freeglut_ext.h 96
glutWireRhombicDodecahedron = platform.createBaseFunction( 
    'glutWireRhombicDodecahedron', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[],
    doc='glutWireRhombicDodecahedron(  ) -> None', 
    argNames=(),
)

# /usr/include/GL/freeglut_ext.h 97
glutSolidRhombicDodecahedron = platform.createBaseFunction( 
    'glutSolidRhombicDodecahedron', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[],
    doc='glutSolidRhombicDodecahedron(  ) -> None', 
    argNames=(),
)

# /usr/include/GL/freeglut_ext.h 98
glutWireSierpinskiSponge = platform.createBaseFunction( 
    'glutWireSierpinskiSponge', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[c_int, arrays.GLdoubleArray, GLdouble],
    doc='glutWireSierpinskiSponge( c_int(num_levels), arrays.GLdoubleArray(offset), GLdouble(scale) ) -> None', 
    argNames=('num_levels', 'offset', 'scale'),
)

glutWireSierpinskiSponge = _wrapper.wrapper( glutWireSierpinskiSponge ).setInputArraySize(
    'offset',
)

# /usr/include/GL/freeglut_ext.h 99
glutSolidSierpinskiSponge = platform.createBaseFunction( 
    'glutSolidSierpinskiSponge', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[c_int, arrays.GLdoubleArray, GLdouble],
    doc='glutSolidSierpinskiSponge( c_int(num_levels), arrays.GLdoubleArray(offset), GLdouble(scale) ) -> None', 
    argNames=('num_levels', 'offset', 'scale'),
)

glutSolidSierpinskiSponge = _wrapper.wrapper( glutSolidSierpinskiSponge ).setInputArraySize(
    'offset',
)

# /usr/include/GL/freeglut_ext.h 100
glutWireCylinder = platform.createBaseFunction( 
    'glutWireCylinder', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[GLdouble, GLdouble, GLint, GLint],
    doc='glutWireCylinder( GLdouble(radius), GLdouble(height), GLint(slices), GLint(stacks) ) -> None', 
    argNames=('radius', 'height', 'slices', 'stacks'),
)

# /usr/include/GL/freeglut_ext.h 101
glutSolidCylinder = platform.createBaseFunction( 
    'glutSolidCylinder', dll=platform.PLATFORM.GLUT, resultType=None, 
    argTypes=[GLdouble, GLdouble, GLint, GLint],
    doc='glutSolidCylinder( GLdouble(radius), GLdouble(height), GLint(slices), GLint(stacks) ) -> None', 
    argNames=('radius', 'height', 'slices', 'stacks'),
)

# /usr/include/GL/freeglut_ext.h 106
glutGetProcAddress = platform.createBaseFunction( 
    'glutGetProcAddress', dll=platform.PLATFORM.GLUT, resultType=c_void_p, 
    argTypes=[c_char_p],
    doc='glutGetProcAddress( STRING(procName) ) -> c_void_p', 
    argNames=('procName',),
)

glutInitContextFlags = platform.createBaseFunction(
    'glutInitContextFlags', dll=platform.PLATFORM.GLUT, resultType=None,
    argTypes=[GLint],
    doc='glutInitContextFlags( GLint(flags) ) -> None',
    argNames = ('flags',),
)
glutInitContextProfile = platform.createBaseFunction(
    'glutInitContextProfile', dll=platform.PLATFORM.GLUT, resultType=None,
    argTypes=[GLint],
    doc='glutInitContextProfile( GLint(profile) ) -> None',
    argNames = ('profile',),
)
glutInitContextVersion = platform.createBaseFunction(
    'glutInitContextVersion', dll=platform.PLATFORM.GLUT, resultType=None,
    argTypes=[GLint,GLint],
    doc='glutInitContextVersion( GLint(majorVersion), GLint(minorVersion) ) -> None',
    argNames = ('majorVersion','minorVersion'),
)
glutFullScreenToggle = platform.createBaseFunction(
    'glutFullScreenToggle', dll=platform.PLATFORM.GLUT, resultType=None,
    argTypes=[],
    doc='glutFullScreenToggle( ) -> None',
    argNames = (),
)

# TODO: this entry point is quite messy, needs a wrapper that creates size, then makes a result 
# object that will de-allocate the memory for the result when finished.  Bleh.
glutGetModeValues = platform.createBaseFunction(
    'glutGetModeValues', dll=platform.PLATFORM.GLUT, resultType=ctypes.POINTER(GLint),
    argTypes=[GLint,ctypes.POINTER(GLint)],
    doc='glutInitContextVersion( GLenum(mode), POINTER(GLint)(size) ) -> POINTER(GLint)',
    argNames = ('mode','size'),
)

fgDeinitialize = platform.createBaseFunction(
    'fgDeinitialize', dll=platform.PLATFORM.GLUT, resultType = None,
    argTypes=[GLint],
    doc ='''fgDeinitialize () -> None
    
Exposed to allow client code to work around an AMD/FGLRX bug on 
GLX platforms. FGLRX and FreeGLUT both register cleanup functions 
that unless registered in the correct order, will cause seg-faults.

To work around this, call fgDeinitialize(False) before doing a 
sys.exit() or similar call that terminates your GLUT mainloop.
''',
)



"""EGL (cross-platform) platform library"""
import ctypes, ctypes.util
from OpenGL.platform import baseplatform, ctypesloader

class EGLPlatform( baseplatform.BasePlatform ):
    """EGL platform for opengl-es only platforms"""
    @baseplatform.lazy_property
    def GLES1(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLESv1_CM', # ick
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def GLES2(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLESv2', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def GLES3(self):
        # implementers guide says to use the same name for the DLL
        return self.GLES2
    @baseplatform.lazy_property
    def GL(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GL', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return self.GLES2 or self.GLES1
    @baseplatform.lazy_property
    def GLU(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLU',
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def GLUT( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'glut', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def OpenGL(self): return self.GL
    
    @baseplatform.lazy_property
    def EGL(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'EGL', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            raise ImportError("Unable to load EGL library", *err.args)
    @baseplatform.lazy_property
    def getExtensionProcedure( self ):
        eglGetProcAddress = self.EGL.eglGetProcAddress
        eglGetProcAddress.restype = ctypes.c_void_p
        return eglGetProcAddress
    @baseplatform.lazy_property
    def GLE( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'gle', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None

    DEFAULT_FUNCTION_TYPE = staticmethod( ctypes.CFUNCTYPE )
    @baseplatform.lazy_property
    def GetCurrentContext( self ):
        return self.EGL.eglGetCurrentContext



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_INTEL_parallel_arrays'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_INTEL_parallel_arrays',error_checker=_errors._error_checker)
GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL=_C('GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL',0x83F7)
GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL=_C('GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL',0x83F6)
GL_PARALLEL_ARRAYS_INTEL=_C('GL_PARALLEL_ARRAYS_INTEL',0x83F4)
GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL=_C('GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL',0x83F8)
GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL=_C('GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL',0x83F5)
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,arrays.GLvoidpArray)
def glColorPointervINTEL(size,type,pointer):pass
@_f
@_p.types(None,_cs.GLenum,arrays.GLvoidpArray)
def glNormalPointervINTEL(type,pointer):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,arrays.GLvoidpArray)
def glTexCoordPointervINTEL(size,type,pointer):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,arrays.GLvoidpArray)
def glVertexPointervINTEL(size,type,pointer):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_EXT_pbuffer'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_EXT_pbuffer',error_checker=_errors._error_checker)
WGL_DRAW_TO_PBUFFER_EXT=_C('WGL_DRAW_TO_PBUFFER_EXT',0x202D)
WGL_MAX_PBUFFER_HEIGHT_EXT=_C('WGL_MAX_PBUFFER_HEIGHT_EXT',0x2030)
WGL_MAX_PBUFFER_PIXELS_EXT=_C('WGL_MAX_PBUFFER_PIXELS_EXT',0x202E)
WGL_MAX_PBUFFER_WIDTH_EXT=_C('WGL_MAX_PBUFFER_WIDTH_EXT',0x202F)
WGL_OPTIMAL_PBUFFER_HEIGHT_EXT=_C('WGL_OPTIMAL_PBUFFER_HEIGHT_EXT',0x2032)
WGL_OPTIMAL_PBUFFER_WIDTH_EXT=_C('WGL_OPTIMAL_PBUFFER_WIDTH_EXT',0x2031)
WGL_PBUFFER_HEIGHT_EXT=_C('WGL_PBUFFER_HEIGHT_EXT',0x2035)
WGL_PBUFFER_LARGEST_EXT=_C('WGL_PBUFFER_LARGEST_EXT',0x2033)
WGL_PBUFFER_WIDTH_EXT=_C('WGL_PBUFFER_WIDTH_EXT',0x2034)
@_f
@_p.types(_cs.HPBUFFEREXT,_cs.HDC,_cs.c_int,_cs.c_int,_cs.c_int,ctypes.POINTER(_cs.c_int))
def wglCreatePbufferEXT(hDC,iPixelFormat,iWidth,iHeight,piAttribList):pass
@_f
@_p.types(_cs.BOOL,_cs.HPBUFFEREXT)
def wglDestroyPbufferEXT(hPbuffer):pass
@_f
@_p.types(_cs.HDC,_cs.HPBUFFEREXT)
def wglGetPbufferDCEXT(hPbuffer):pass
@_f
@_p.types(_cs.BOOL,_cs.HPBUFFEREXT,_cs.c_int,ctypes.POINTER(_cs.c_int))
def wglQueryPbufferEXT(hPbuffer,iAttribute,piValue):pass
@_f
@_p.types(_cs.c_int,_cs.HPBUFFEREXT,_cs.HDC)
def wglReleasePbufferDCEXT(hPbuffer,hDC):pass



'''OpenGL extension INGR.interlace_read

This module customises the behaviour of the 
OpenGL.raw.GL.INGR.interlace_read to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides a way to skip rows of pixels when reading
	or copying pixel rectangles.  This extension is complementary to
	the EXT_interlace extension except that it has no affect on getting
	texture images.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/INGR/interlace_read.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.INGR.interlace_read import *
from OpenGL.raw.GL.INGR.interlace_read import _EXTENSION_NAME

def glInitInterlaceReadINGR():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension ATI.pixel_format_float

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.pixel_format_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds pixel formats with floating-point RGBA color
	components.
	
	The size of each float components is specified using the same
	WGL_RED_BITS_ARB, WGL_GREEN_BITS_ARB, WGL_BLUE_BITS_ARB and
	WGL_ALPHA_BITS_ARB pixel format attributes that are used for
	defining the size of fixed-point components.  32 bit floating-
	point components are in the standard IEEE float format.  16 bit
	floating-point components have 1 sign bit, 5 exponent bits,
	and 10 mantissa bits. 
	
	In standard OpenGL RGBA color components are normally clamped to
	the range [0,1].  The color components of a float buffer are
	clamped to the limits of the range representable by their format.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/pixel_format_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.pixel_format_float import *
from OpenGL.raw.GL.ATI.pixel_format_float import _EXTENSION_NAME

def glInitPixelFormatFloatATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_coordinate_frame'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_coordinate_frame',error_checker=_errors._error_checker)
GL_BINORMAL_ARRAY_EXT=_C('GL_BINORMAL_ARRAY_EXT',0x843A)
GL_BINORMAL_ARRAY_POINTER_EXT=_C('GL_BINORMAL_ARRAY_POINTER_EXT',0x8443)
GL_BINORMAL_ARRAY_STRIDE_EXT=_C('GL_BINORMAL_ARRAY_STRIDE_EXT',0x8441)
GL_BINORMAL_ARRAY_TYPE_EXT=_C('GL_BINORMAL_ARRAY_TYPE_EXT',0x8440)
GL_CURRENT_BINORMAL_EXT=_C('GL_CURRENT_BINORMAL_EXT',0x843C)
GL_CURRENT_TANGENT_EXT=_C('GL_CURRENT_TANGENT_EXT',0x843B)
GL_MAP1_BINORMAL_EXT=_C('GL_MAP1_BINORMAL_EXT',0x8446)
GL_MAP1_TANGENT_EXT=_C('GL_MAP1_TANGENT_EXT',0x8444)
GL_MAP2_BINORMAL_EXT=_C('GL_MAP2_BINORMAL_EXT',0x8447)
GL_MAP2_TANGENT_EXT=_C('GL_MAP2_TANGENT_EXT',0x8445)
GL_TANGENT_ARRAY_EXT=_C('GL_TANGENT_ARRAY_EXT',0x8439)
GL_TANGENT_ARRAY_POINTER_EXT=_C('GL_TANGENT_ARRAY_POINTER_EXT',0x8442)
GL_TANGENT_ARRAY_STRIDE_EXT=_C('GL_TANGENT_ARRAY_STRIDE_EXT',0x843F)
GL_TANGENT_ARRAY_TYPE_EXT=_C('GL_TANGENT_ARRAY_TYPE_EXT',0x843E)
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glBinormal3bEXT(bx,by,bz):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glBinormal3bvEXT(v):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glBinormal3dEXT(bx,by,bz):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glBinormal3dvEXT(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glBinormal3fEXT(bx,by,bz):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glBinormal3fvEXT(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glBinormal3iEXT(bx,by,bz):pass
@_f
@_p.types(None,arrays.GLintArray)
def glBinormal3ivEXT(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glBinormal3sEXT(bx,by,bz):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glBinormal3svEXT(v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glBinormalPointerEXT(type,stride,pointer):pass
@_f
@_p.types(None,_cs.GLbyte,_cs.GLbyte,_cs.GLbyte)
def glTangent3bEXT(tx,ty,tz):pass
@_f
@_p.types(None,arrays.GLbyteArray)
def glTangent3bvEXT(v):pass
@_f
@_p.types(None,_cs.GLdouble,_cs.GLdouble,_cs.GLdouble)
def glTangent3dEXT(tx,ty,tz):pass
@_f
@_p.types(None,arrays.GLdoubleArray)
def glTangent3dvEXT(v):pass
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glTangent3fEXT(tx,ty,tz):pass
@_f
@_p.types(None,arrays.GLfloatArray)
def glTangent3fvEXT(v):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLint,_cs.GLint)
def glTangent3iEXT(tx,ty,tz):pass
@_f
@_p.types(None,arrays.GLintArray)
def glTangent3ivEXT(v):pass
@_f
@_p.types(None,_cs.GLshort,_cs.GLshort,_cs.GLshort)
def glTangent3sEXT(tx,ty,tz):pass
@_f
@_p.types(None,arrays.GLshortArray)
def glTangent3svEXT(v):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,ctypes.c_void_p)
def glTangentPointerEXT(type,stride,pointer):pass



'''OpenGL extension NV.path_rendering

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.path_rendering to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/path_rendering.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.path_rendering import *
from OpenGL.raw.GLES2.NV.path_rendering import _EXTENSION_NAME

def glInitPathRenderingNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glPathCommandsNV.commands size not checked against numCommands
# INPUT glPathCommandsNV.coords size not checked against 'numCoords,coordType'
glPathCommandsNV=wrapper.wrapper(glPathCommandsNV).setInputArraySize(
    'commands', None
).setInputArraySize(
    'coords', None
)
# INPUT glPathCoordsNV.coords size not checked against 'numCoords,coordType'
glPathCoordsNV=wrapper.wrapper(glPathCoordsNV).setInputArraySize(
    'coords', None
)
# INPUT glPathSubCommandsNV.commands size not checked against numCommands
# INPUT glPathSubCommandsNV.coords size not checked against 'numCoords,coordType'
glPathSubCommandsNV=wrapper.wrapper(glPathSubCommandsNV).setInputArraySize(
    'commands', None
).setInputArraySize(
    'coords', None
)
# INPUT glPathSubCoordsNV.coords size not checked against 'numCoords,coordType'
glPathSubCoordsNV=wrapper.wrapper(glPathSubCoordsNV).setInputArraySize(
    'coords', None
)
# INPUT glPathStringNV.pathString size not checked against length
glPathStringNV=wrapper.wrapper(glPathStringNV).setInputArraySize(
    'pathString', None
)
# INPUT glPathGlyphsNV.charcodes size not checked against 'numGlyphs,type,charcodes'
# INPUT glPathGlyphsNV.fontName size not checked against 'fontTarget,fontName'
glPathGlyphsNV=wrapper.wrapper(glPathGlyphsNV).setInputArraySize(
    'charcodes', None
).setInputArraySize(
    'fontName', None
)
# INPUT glPathGlyphRangeNV.fontName size not checked against 'fontTarget,fontName'
glPathGlyphRangeNV=wrapper.wrapper(glPathGlyphRangeNV).setInputArraySize(
    'fontName', None
)
# INPUT glWeightPathsNV.paths size not checked against numPaths
# INPUT glWeightPathsNV.weights size not checked against numPaths
glWeightPathsNV=wrapper.wrapper(glWeightPathsNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'weights', None
)
# INPUT glTransformPathNV.transformValues size not checked against 'transformType'
glTransformPathNV=wrapper.wrapper(glTransformPathNV).setInputArraySize(
    'transformValues', None
)
# INPUT glPathParameterivNV.value size not checked against 'pname'
glPathParameterivNV=wrapper.wrapper(glPathParameterivNV).setInputArraySize(
    'value', None
)
# INPUT glPathParameterfvNV.value size not checked against 'pname'
glPathParameterfvNV=wrapper.wrapper(glPathParameterfvNV).setInputArraySize(
    'value', None
)
# INPUT glPathDashArrayNV.dashArray size not checked against dashCount
glPathDashArrayNV=wrapper.wrapper(glPathDashArrayNV).setInputArraySize(
    'dashArray', None
)
# INPUT glStencilFillPathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glStencilFillPathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glStencilFillPathInstancedNV=wrapper.wrapper(glStencilFillPathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
# INPUT glStencilStrokePathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glStencilStrokePathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glStencilStrokePathInstancedNV=wrapper.wrapper(glStencilStrokePathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
# INPUT glCoverFillPathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glCoverFillPathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glCoverFillPathInstancedNV=wrapper.wrapper(glCoverFillPathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
# INPUT glCoverStrokePathInstancedNV.paths size not checked against 'numPaths,pathNameType,paths'
# INPUT glCoverStrokePathInstancedNV.transformValues size not checked against 'numPaths,transformType'
glCoverStrokePathInstancedNV=wrapper.wrapper(glCoverStrokePathInstancedNV).setInputArraySize(
    'paths', None
).setInputArraySize(
    'transformValues', None
)
glGetPathParameterivNV=wrapper.wrapper(glGetPathParameterivNV).setOutput(
    'value',size=(4,),orPassIn=True
)
glGetPathParameterfvNV=wrapper.wrapper(glGetPathParameterfvNV).setOutput(
    'value',size=(4,),orPassIn=True
)
glGetPathCommandsNV=wrapper.wrapper(glGetPathCommandsNV).setOutput(
    'commands',size=_glgets._glget_size_mapping,pnameArg='path',orPassIn=True
)
glGetPathCoordsNV=wrapper.wrapper(glGetPathCoordsNV).setOutput(
    'coords',size=_glgets._glget_size_mapping,pnameArg='path',orPassIn=True
)
glGetPathDashArrayNV=wrapper.wrapper(glGetPathDashArrayNV).setOutput(
    'dashArray',size=_glgets._glget_size_mapping,pnameArg='path',orPassIn=True
)
# OUTPUT glGetPathMetricsNV.metrics COMPSIZE(metricQueryMask, numPaths, stride) 
# INPUT glGetPathMetricsNV.paths size not checked against 'numPaths,pathNameType,paths'
glGetPathMetricsNV=wrapper.wrapper(glGetPathMetricsNV).setInputArraySize(
    'paths', None
)
# OUTPUT glGetPathMetricRangeNV.metrics COMPSIZE(metricQueryMask, numPaths, stride) 
# INPUT glGetPathSpacingNV.paths size not checked against 'numPaths,pathNameType,paths'
# OUTPUT glGetPathSpacingNV.returnedSpacing COMPSIZE(pathListMode, numPaths) 
glGetPathSpacingNV=wrapper.wrapper(glGetPathSpacingNV).setInputArraySize(
    'paths', None
)
glPointAlongPathNV=wrapper.wrapper(glPointAlongPathNV).setOutput(
    'tangentX',size=(1,),orPassIn=True
).setOutput(
    'tangentY',size=(1,),orPassIn=True
).setOutput(
    'x',size=(1,),orPassIn=True
).setOutput(
    'y',size=(1,),orPassIn=True
)
# INPUT glPathColorGenNV.coeffs size not checked against 'genMode,colorFormat'
glPathColorGenNV=wrapper.wrapper(glPathColorGenNV).setInputArraySize(
    'coeffs', None
)
# INPUT glPathTexGenNV.coeffs size not checked against 'genMode,components'
glPathTexGenNV=wrapper.wrapper(glPathTexGenNV).setInputArraySize(
    'coeffs', None
)
glGetPathColorGenivNV=wrapper.wrapper(glGetPathColorGenivNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetPathColorGenfvNV=wrapper.wrapper(glGetPathColorGenfvNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetPathTexGenivNV=wrapper.wrapper(glGetPathTexGenivNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetPathTexGenfvNV=wrapper.wrapper(glGetPathTexGenfvNV).setOutput(
    'value',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.blend_color

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.blend_color to provide a more 
Python-friendly API

Overview (from the spec)
	
	Blending capability is extended by defining a constant color that can
	be included in blending equations.  A typical usage is blending two
	RGB images.  Without the constant blend factor, one image must have
	an alpha channel with each pixel set to the desired blend factor.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/blend_color.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.blend_color import *
from OpenGL.raw.GL.EXT.blend_color import _EXTENSION_NAME

def glInitBlendColorEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_APPLE_flush_buffer_range'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_APPLE_flush_buffer_range',error_checker=_errors._error_checker)
GL_BUFFER_FLUSHING_UNMAP_APPLE=_C('GL_BUFFER_FLUSHING_UNMAP_APPLE',0x8A13)
GL_BUFFER_SERIALIZED_MODIFY_APPLE=_C('GL_BUFFER_SERIALIZED_MODIFY_APPLE',0x8A12)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glBufferParameteriAPPLE(target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLintptr,_cs.GLsizeiptr)
def glFlushMappedBufferRangeAPPLE(target,offset,size):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_debug_label'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_debug_label',error_checker=_errors._error_checker)
GL_BUFFER_OBJECT_EXT=_C('GL_BUFFER_OBJECT_EXT',0x9151)
GL_PROGRAM_OBJECT_EXT=_C('GL_PROGRAM_OBJECT_EXT',0x8B40)
GL_PROGRAM_PIPELINE_OBJECT_EXT=_C('GL_PROGRAM_PIPELINE_OBJECT_EXT',0x8A4F)
GL_QUERY_OBJECT_EXT=_C('GL_QUERY_OBJECT_EXT',0x9153)
GL_SAMPLER=_C('GL_SAMPLER',0x82E6)
GL_SHADER_OBJECT_EXT=_C('GL_SHADER_OBJECT_EXT',0x8B48)
GL_TRANSFORM_FEEDBACK=_C('GL_TRANSFORM_FEEDBACK',0x8E22)
GL_VERTEX_ARRAY_OBJECT_EXT=_C('GL_VERTEX_ARRAY_OBJECT_EXT',0x9154)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetObjectLabelEXT(type,object,bufSize,length,label):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint,_cs.GLsizei,arrays.GLcharArray)
def glLabelObjectEXT(type,object,length,label):pass



"""Simplistic wrapper decorator for Python-coded wrappers"""
from OpenGL.latebind import Curry
from OpenGL import MODULE_ANNOTATIONS

class _LazyWrapper( Curry ):
    """Marker to tell us that an object is a lazy wrapper"""

def lazy( baseFunction ):
    """Produce a lazy-binding decorator that uses baseFunction

    Allows simple implementation of wrappers where the
    whole of the wrapper can be summed up as do 1 thing
    then call base function with the cleaned up result.

    Passes baseFunction in as the first argument of the
    wrapped function, all other parameters are passed
    unchanged.  The wrapper class created has __nonzero__
    and similar common wrapper entry points defined.
    """
    def wrap( wrapper ):
        """Wrap wrapper with baseFunction"""
        def __nonzero__( self ):
            return bool( baseFunction )
        def __repr__( self ):
            return '%s( %r )'%(
                'OpenGL.lazywrapper.lazy',
                baseFunction.__name__,
            )
        _with_wrapper = type( wrapper.__name__, (_LazyWrapper,), {
            '__repr__': __repr__,
            '__doc__': wrapper.__doc__,
            '__nonzero__': __nonzero__,
            'wrappedOperation': baseFunction,
            'restype': getattr(wrapper, 'restype',getattr(baseFunction,'restype',None)),
        } )
        with_wrapper = _with_wrapper(wrapper,baseFunction)
        with_wrapper.__name__ = wrapper.__name__
        if hasattr( baseFunction, '__module__' ):
            with_wrapper.__module__ = baseFunction.__module__
        return with_wrapper
    return wrap


if __name__ == "__main__":
    from OpenGL.raw import GLU
    func = GLU.gluNurbsCallbackData
    output = []
    def testwrap( base ):
        "Testing"
        output.append( base )
    testlazy = lazy( func )( testwrap )
    testlazy( )
    assert testlazy.__doc__ == "Testing"
    assert testlazy.__class__.__name__ == 'testwrap'
    assert testlazy.__name__ == 'testwrap'
    assert testlazy.baseFunction is func
    assert testlazy.wrapperFunction is testwrap
    assert output



'''OpenGL extension NV.internalformat_sample_query

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.internalformat_sample_query to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/internalformat_sample_query.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.internalformat_sample_query import *
from OpenGL.raw.GLES2.NV.internalformat_sample_query import _EXTENSION_NAME

def glInitInternalformatSampleQueryNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetInternalformatSampleivNV.params size not checked against bufSize
glGetInternalformatSampleivNV=wrapper.wrapper(glGetInternalformatSampleivNV).setInputArraySize(
    'params', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.draw_range_elements

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.draw_range_elements to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/draw_range_elements.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.draw_range_elements import *
from OpenGL.raw.GL.EXT.draw_range_elements import _EXTENSION_NAME

def glInitDrawRangeElementsEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawRangeElementsEXT.indices size not checked against 'count,type'
glDrawRangeElementsEXT=wrapper.wrapper(glDrawRangeElementsEXT).setInputArraySize(
    'indices', None
)
### END AUTOGENERATED SECTION

glDrawRangeElementsEXT = wrapper.wrapper( glDrawRangeElementsEXT ).setPyConverter(
    'indices', arrays.AsArrayOfType( 'indices', 'type' ),
).setReturnValues( 
    wrapper.returnPyArgument( 'indices' ) 
)


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.EGL import _types as _cs
# End users want this...
from OpenGL.raw.EGL._types import *
from OpenGL.raw.EGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'EGL_VERSION_EGL_1_4'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.EGL,'EGL_VERSION_EGL_1_4',error_checker=_errors._error_checker)
# EGL_DEFAULT_DISPLAY=_C('EGL_DEFAULT_DISPLAY',((EGLNativeDisplayType)0))
EGL_MULTISAMPLE_RESOLVE=_C('EGL_MULTISAMPLE_RESOLVE',0x3099)
EGL_MULTISAMPLE_RESOLVE_BOX=_C('EGL_MULTISAMPLE_RESOLVE_BOX',0x309B)
EGL_MULTISAMPLE_RESOLVE_BOX_BIT=_C('EGL_MULTISAMPLE_RESOLVE_BOX_BIT',0x0200)
EGL_MULTISAMPLE_RESOLVE_DEFAULT=_C('EGL_MULTISAMPLE_RESOLVE_DEFAULT',0x309A)
EGL_OPENGL_API=_C('EGL_OPENGL_API',0x30A2)
EGL_OPENGL_BIT=_C('EGL_OPENGL_BIT',0x0008)
EGL_SWAP_BEHAVIOR_PRESERVED_BIT=_C('EGL_SWAP_BEHAVIOR_PRESERVED_BIT',0x0400)
@_f
@_p.types(_cs.EGLContext,)
def eglGetCurrentContext():pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_AMD_vertex_shader_tessellator'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_AMD_vertex_shader_tessellator',error_checker=_errors._error_checker)
GL_CONTINUOUS_AMD=_C('GL_CONTINUOUS_AMD',0x9007)
GL_DISCRETE_AMD=_C('GL_DISCRETE_AMD',0x9006)
GL_INT_SAMPLER_BUFFER_AMD=_C('GL_INT_SAMPLER_BUFFER_AMD',0x9002)
GL_SAMPLER_BUFFER_AMD=_C('GL_SAMPLER_BUFFER_AMD',0x9001)
GL_TESSELLATION_FACTOR_AMD=_C('GL_TESSELLATION_FACTOR_AMD',0x9005)
GL_TESSELLATION_MODE_AMD=_C('GL_TESSELLATION_MODE_AMD',0x9004)
GL_UNSIGNED_INT_SAMPLER_BUFFER_AMD=_C('GL_UNSIGNED_INT_SAMPLER_BUFFER_AMD',0x9003)
@_f
@_p.types(None,_cs.GLfloat)
def glTessellationFactorAMD(factor):pass
@_f
@_p.types(None,_cs.GLenum)
def glTessellationModeAMD(mode):pass



'''OpenGL extension ARB.vertex_type_2_10_10_10_rev

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.vertex_type_2_10_10_10_rev to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds the following data formats:
	
	Two new vertex attribute data formats: a signed 2.10.10.10 and an
	unsigned 2.10.10.10 vertex data format. These vertex data formats
	describe a 4 component stream which can be used to store normals or
	other attributes in a quantized form. Normals, tangents, binormals
	and other vertex attributes can often be specified at reduced
	precision without introducing noticeable artifacts, reducing the
	amount of memory and memory bandwidth they consume.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/vertex_type_2_10_10_10_rev.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.vertex_type_2_10_10_10_rev import *
from OpenGL.raw.GL.ARB.vertex_type_2_10_10_10_rev import _EXTENSION_NAME

def glInitVertexType2101010RevARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertexAttribP1uiv=wrapper.wrapper(glVertexAttribP1uiv).setInputArraySize(
    'value', 1
)
glVertexAttribP2uiv=wrapper.wrapper(glVertexAttribP2uiv).setInputArraySize(
    'value', 1
)
glVertexAttribP3uiv=wrapper.wrapper(glVertexAttribP3uiv).setInputArraySize(
    'value', 1
)
glVertexAttribP4uiv=wrapper.wrapper(glVertexAttribP4uiv).setInputArraySize(
    'value', 1
)
glVertexP2uiv=wrapper.wrapper(glVertexP2uiv).setInputArraySize(
    'value', 1
)
glVertexP3uiv=wrapper.wrapper(glVertexP3uiv).setInputArraySize(
    'value', 1
)
glVertexP4uiv=wrapper.wrapper(glVertexP4uiv).setInputArraySize(
    'value', 1
)
glTexCoordP1uiv=wrapper.wrapper(glTexCoordP1uiv).setInputArraySize(
    'coords', 1
)
glTexCoordP2uiv=wrapper.wrapper(glTexCoordP2uiv).setInputArraySize(
    'coords', 1
)
glTexCoordP3uiv=wrapper.wrapper(glTexCoordP3uiv).setInputArraySize(
    'coords', 1
)
glTexCoordP4uiv=wrapper.wrapper(glTexCoordP4uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP1uiv=wrapper.wrapper(glMultiTexCoordP1uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP2uiv=wrapper.wrapper(glMultiTexCoordP2uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP3uiv=wrapper.wrapper(glMultiTexCoordP3uiv).setInputArraySize(
    'coords', 1
)
glMultiTexCoordP4uiv=wrapper.wrapper(glMultiTexCoordP4uiv).setInputArraySize(
    'coords', 1
)
glNormalP3uiv=wrapper.wrapper(glNormalP3uiv).setInputArraySize(
    'coords', 1
)
glColorP3uiv=wrapper.wrapper(glColorP3uiv).setInputArraySize(
    'color', 1
)
glColorP4uiv=wrapper.wrapper(glColorP4uiv).setInputArraySize(
    'color', 1
)
glSecondaryColorP3uiv=wrapper.wrapper(glSecondaryColorP3uiv).setInputArraySize(
    'color', 1
)
### END AUTOGENERATED SECTION


'''OpenGL extension ATI.envmap_bumpmap

This module customises the behaviour of the 
OpenGL.raw.GL.ATI.envmap_bumpmap to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds environment mapped bump mapping (EMBM) to the GL.
	The method exposed by this extension is to use a dependent texture
	read on a bumpmap (du,dv) texture to offset the texture coordinates
	read into a map on another texture unit.  This (du,dv) offset is also 
	rotated through a user-specified rotation matrix to get the texture 
	coordinates into the appropriate space.
	
	A new texture format is introduced in order for specifying the (du,dv)
	bumpmap texture.  This map represents -1 <= du,dv <= 1 offsets to
	be applied to the texture coordinates used to read into the base
	map.  Additionally, the (du,dv) offsets are transformed by a rotation
	matrix that this extension allows the user to specify.  Further, a 
	new color operation is added to EXT_texture_env_combine to specify 
	both that bumpmapping is enabled and which texture unit to apply 
	the bump offset to.  

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ATI/envmap_bumpmap.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ATI.envmap_bumpmap import *
from OpenGL.raw.GL.ATI.envmap_bumpmap import _EXTENSION_NAME

def glInitEnvmapBumpmapATI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glTexBumpParameterivATI.param size not checked against 'pname'
glTexBumpParameterivATI=wrapper.wrapper(glTexBumpParameterivATI).setInputArraySize(
    'param', None
)
# INPUT glTexBumpParameterfvATI.param size not checked against 'pname'
glTexBumpParameterfvATI=wrapper.wrapper(glTexBumpParameterfvATI).setInputArraySize(
    'param', None
)
glGetTexBumpParameterivATI=wrapper.wrapper(glGetTexBumpParameterivATI).setOutput(
    'param',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetTexBumpParameterfvATI=wrapper.wrapper(glGetTexBumpParameterfvATI).setOutput(
    'param',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION


"""OSMesa-specific features

To request an OSMesa context, you need to run your script with:

    PYOPENGL_PLATFORM=osmesa

defined in your shell/execution environment.
"""
import ctypes, ctypes.util
from OpenGL.platform import baseplatform, ctypesloader
from OpenGL.constant import Constant
from OpenGL.raw.osmesa import _types

class OSMesaPlatform( baseplatform.BasePlatform ):
    """OSMesa implementation for PyOpenGL"""
    EXPORTED_NAMES = baseplatform.BasePlatform.EXPORTED_NAMES[:] + [
        'OSMesa',
    ]
    @baseplatform.lazy_property
    def GL(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'OSMesa', 
                mode=ctypes.RTLD_GLOBAL 
            ) 
        except OSError as err:
            raise ImportError("Unable to load OpenGL library", *err.args)
    @baseplatform.lazy_property
    def GLU(self):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'GLU',
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def GLUT( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'glut', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def GLE( self ):
        try:
            return ctypesloader.loadLibrary(
                ctypes.cdll,
                'gle', 
                mode=ctypes.RTLD_GLOBAL 
            )
        except OSError as err:
            return None
    @baseplatform.lazy_property
    def OSMesa( self ): return self.GL
        
    DEFAULT_FUNCTION_TYPE = staticmethod( ctypes.CFUNCTYPE )

    @baseplatform.lazy_property
    def GetCurrentContext( self ):
        function = self.OSMesa.OSMesaGetCurrentContext
        function.restype = _types.OSMesaContext
        return function
    @baseplatform.lazy_property
    def CurrentContextIsValid( self ): return self.GetCurrentContext
    
    @baseplatform.lazy_property
    def getExtensionProcedure( self ):
        function = self.OSMesa.OSMesaGetProcAddress
        function.restype = ctypes.c_void_p
        return function

    def getGLUTFontPointer( self, constant ):
        """Platform specific function to retrieve a GLUT font pointer
        
        GLUTAPI void *glutBitmap9By15;
        #define GLUT_BITMAP_9_BY_15		(&glutBitmap9By15)
        
        Key here is that we want the addressof the pointer in the DLL,
        not the pointer in the DLL.  That is, our pointer is to the 
        pointer defined in the DLL, we don't want the *value* stored in
        that pointer.
        """
        name = [ x.title() for x in constant.split( '_' )[1:] ]
        internal = 'glut' + "".join( [x.title() for x in name] )
        pointer = ctypes.c_void_p.in_dll( self.GLUT, internal )
        return ctypes.c_void_p(ctypes.addressof(pointer))



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_IBM_vertex_array_lists'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_IBM_vertex_array_lists',error_checker=_errors._error_checker)
GL_COLOR_ARRAY_LIST_IBM=_C('GL_COLOR_ARRAY_LIST_IBM',103072)
GL_COLOR_ARRAY_LIST_STRIDE_IBM=_C('GL_COLOR_ARRAY_LIST_STRIDE_IBM',103082)
GL_EDGE_FLAG_ARRAY_LIST_IBM=_C('GL_EDGE_FLAG_ARRAY_LIST_IBM',103075)
GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM=_C('GL_EDGE_FLAG_ARRAY_LIST_STRIDE_IBM',103085)
GL_FOG_COORDINATE_ARRAY_LIST_IBM=_C('GL_FOG_COORDINATE_ARRAY_LIST_IBM',103076)
GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM=_C('GL_FOG_COORDINATE_ARRAY_LIST_STRIDE_IBM',103086)
GL_INDEX_ARRAY_LIST_IBM=_C('GL_INDEX_ARRAY_LIST_IBM',103073)
GL_INDEX_ARRAY_LIST_STRIDE_IBM=_C('GL_INDEX_ARRAY_LIST_STRIDE_IBM',103083)
GL_NORMAL_ARRAY_LIST_IBM=_C('GL_NORMAL_ARRAY_LIST_IBM',103071)
GL_NORMAL_ARRAY_LIST_STRIDE_IBM=_C('GL_NORMAL_ARRAY_LIST_STRIDE_IBM',103081)
GL_SECONDARY_COLOR_ARRAY_LIST_IBM=_C('GL_SECONDARY_COLOR_ARRAY_LIST_IBM',103077)
GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM=_C('GL_SECONDARY_COLOR_ARRAY_LIST_STRIDE_IBM',103087)
GL_TEXTURE_COORD_ARRAY_LIST_IBM=_C('GL_TEXTURE_COORD_ARRAY_LIST_IBM',103074)
GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM=_C('GL_TEXTURE_COORD_ARRAY_LIST_STRIDE_IBM',103084)
GL_VERTEX_ARRAY_LIST_IBM=_C('GL_VERTEX_ARRAY_LIST_IBM',103070)
GL_VERTEX_ARRAY_LIST_STRIDE_IBM=_C('GL_VERTEX_ARRAY_LIST_STRIDE_IBM',103080)
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLint,arrays.GLvoidpArray,_cs.GLint)
def glColorPointerListIBM(size,type,stride,pointer,ptrstride):pass
@_f
@_p.types(None,_cs.GLint,arrays.GLbooleanArray,_cs.GLint)
def glEdgeFlagPointerListIBM(stride,pointer,ptrstride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,arrays.GLvoidpArray,_cs.GLint)
def glFogCoordPointerListIBM(type,stride,pointer,ptrstride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,arrays.GLvoidpArray,_cs.GLint)
def glIndexPointerListIBM(type,stride,pointer,ptrstride):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,arrays.GLvoidpArray,_cs.GLint)
def glNormalPointerListIBM(type,stride,pointer,ptrstride):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLint,arrays.GLvoidpArray,_cs.GLint)
def glSecondaryColorPointerListIBM(size,type,stride,pointer,ptrstride):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLint,arrays.GLvoidpArray,_cs.GLint)
def glTexCoordPointerListIBM(size,type,stride,pointer,ptrstride):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLenum,_cs.GLint,arrays.GLvoidpArray,_cs.GLint)
def glVertexPointerListIBM(size,type,stride,pointer,ptrstride):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_ARB_create_context_profile'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_ARB_create_context_profile',error_checker=_errors._error_checker)
ERROR_INVALID_PROFILE_ARB=_C('ERROR_INVALID_PROFILE_ARB',0x2096)
WGL_CONTEXT_COMPATIBILITY_PROFILE_BIT_ARB=_C('WGL_CONTEXT_COMPATIBILITY_PROFILE_BIT_ARB',0x00000002)
WGL_CONTEXT_CORE_PROFILE_BIT_ARB=_C('WGL_CONTEXT_CORE_PROFILE_BIT_ARB',0x00000001)
WGL_CONTEXT_PROFILE_MASK_ARB=_C('WGL_CONTEXT_PROFILE_MASK_ARB',0x9126)




'''OpenGL extension OES.query_matrix

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.query_matrix to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/query_matrix.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.query_matrix import *
from OpenGL.raw.GLES1.OES.query_matrix import _EXTENSION_NAME

def glInitQueryMatrixOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glQueryMatrixxOES=wrapper.wrapper(glQueryMatrixxOES).setOutput(
    'exponent',size=(16,),orPassIn=True
).setOutput(
    'mantissa',size=(16,),orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.histogram

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.histogram to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension defines pixel operations that count occurences of
	specific color component values (histogram) and that track the minimum
	and maximum color component values (minmax).  An optional mode allows
	pixel data to be discarded after the histogram and/or minmax operations
	are completed.  Otherwise the pixel data continue on to the next
	operation unaffected.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/histogram.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.histogram import *
from OpenGL.raw.GL.EXT.histogram import _EXTENSION_NAME

def glInitHistogramEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# OUTPUT glGetHistogramEXT.values COMPSIZE(target, format, type) 
glGetHistogramParameterfvEXT=wrapper.wrapper(glGetHistogramParameterfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetHistogramParameterivEXT=wrapper.wrapper(glGetHistogramParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
# OUTPUT glGetMinmaxEXT.values COMPSIZE(target, format, type) 
glGetMinmaxParameterfvEXT=wrapper.wrapper(glGetMinmaxParameterfvEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
glGetMinmaxParameterivEXT=wrapper.wrapper(glGetMinmaxParameterivEXT).setOutput(
    'params',size=_glgets._glget_size_mapping,pnameArg='pname',orPassIn=True
)
### END AUTOGENERATED SECTION

glGetHistogramParameterfvEXT = wrapper.wrapper(glGetHistogramParameterfvEXT).setOutput(
    "params",(1,), orPassIn=True
)
glGetHistogramParameterivEXT = wrapper.wrapper(glGetHistogramParameterivEXT).setOutput(
    "params",(1,), orPassIn=True
)



"""GLUT functions requiring special handling to provide Pythonic wrappers

Note:
    GLUT callbacks are controlled by a flag in the platform module.  The
    GLUT_GUARD_CALLBACKS flag controls whether to wrap passed functions
    with error-checking and context-validity checking code so that the
    callbacks will only trigger if there is a valid context.  This is done
    so that systems such as Win32 will not continue running GLUT callbacks
    after the system has exited.

    Note:
        This is not a problem with FreeGLUT on Linux, so Linux does not
        add the extra overhead of the wrapper function.
    Note:
        This hack does *not* prevent hanging if there is no GLUT callback
        being triggered.  I.e. if you create a GLUT program that doesn't
        explicitly call exit and doesn't call display or the like in a timer
        then your app will hang on exit on Win32.

XXX the platform-specific stuff should be getting done in the 
platform module *not* in the module here!
"""
from OpenGL.platform import CurrentContextIsValid, GLUT_GUARD_CALLBACKS, PLATFORM
GLUT = PLATFORM.GLUT
from OpenGL import contextdata, error, platform, logs
from OpenGL.raw import GLUT as _simple
from OpenGL._bytes import bytes, unicode,as_8_bit
import ctypes, os, sys, traceback
PLATFORM = platform.PLATFORM
FUNCTION_TYPE = _simple.CALLBACK_FUNCTION_TYPE
from OpenGL._bytes import long, integer_types

_log = logs.getLog( 'OpenGL.GLUT.special' )

if os.name == "nt":
    _log.info( """Using NT-specific GLUT calls with exit callbacks""" )
    _exitfunctype = FUNCTION_TYPE( None, ctypes.c_int )
    __glutInitWithExit = platform.createBaseFunction(
        '__glutInitWithExit', dll=platform.PLATFORM.GLUT, resultType=None,
        argTypes=[ctypes.POINTER(ctypes.c_int),ctypes.POINTER(ctypes.c_char_p),_exitfunctype],
        doc='glutInit( POINTER(c_int)(pargc), POINTER(STRING)(argv) ) -> None',
        argNames=('pargc', 'argv'),
    )
    __glutCreateWindowWithExit = platform.createBaseFunction(
        '__glutCreateWindowWithExit', dll=platform.PLATFORM.GLUT, resultType=ctypes.c_int,
        argTypes=[ctypes.c_char_p,_exitfunctype],
        doc='glutCreateWindow( STRING(title) ) -> c_int',
        argNames=('title',),
    )
    __glutCreateMenuWithExit = platform.createBaseFunction( 
        '__glutCreateMenuWithExit', dll=platform.PLATFORM.GLUT, resultType=ctypes.c_int, 
        argTypes=[FUNCTION_TYPE(None, ctypes.c_int),_exitfunctype],
        doc='glutCreateMenu( FUNCTION_TYPE(None, c_int)(callback) ) -> c_int', 
        argNames=('callback',),
    )
else:
    # Linux, OSX, etceteras
    __glutInitWithExit = None
if __glutInitWithExit:
    # Normal platforms + Win32 w/ FreeGLUT (SF#2813722)
    import sys
    _exitfunc = _exitfunctype(sys.exit)
    
    def _base_glutInit(pargc, argv):
        """Overrides base glut init with exit-function-aware version"""
        return __glutInitWithExit(pargc, argv, _exitfunc)
    def glutCreateWindow(title):
        """Create window with given title
        
        This is the Win32-specific version that handles
        registration of an exit-function handler 
        """
        return __glutCreateWindowWithExit(title, _exitfunc)
    def glutCreateMenu(callback):
        """Create menu with given callback 
        
        This is the Win32-specific version that handles 
        registration of an exit-function callback.
        """
        return __glutCreateMenuWithExit(callback, _exitfunc)
else:
    _base_glutInit = platform.nullFunction( 
        'glutInit', GLUT,
        resultType=None, 
        argTypes=(ctypes.POINTER(ctypes.c_int),ctypes.POINTER( ctypes.c_char_p )),
        doc = 'Initialize the GLUT library', argNames = ('argcp','argv'),
        module = __name__,
        error_checker = None,
    )
##_base_glutDisplayFunc = GLUT.glutDisplayFunc
##_base_glutIdleFunc = GLUT.glutIdleFunc
##_base_glutEntryFunc = GLUT.glutEntryFunc
##_base_glutReshapeFunc = GLUT.glutReshapeFunc
_base_glutDestroyWindow = getattr(GLUT, 'glutDestroyWindow', None)

class GLUTCallback( object ):
    """Class implementing GLUT Callback registration functions"""
    def __init__( self, typeName, parameterTypes, parameterNames ):
        """Initialise the glut callback instance"""
        self.typeName = typeName
        def describe( typ, name ):
            return '(int) %s'%(name)
        self.__doc__ = """Specify handler for GLUT %r events
    def handler( %s ):
        return None"""%( typeName, ", ".join([
            describe( typ,name )
            for (typ,name) in zip( parameterTypes, parameterNames )
        ]))
        try:
            self.wrappedOperation = getattr( GLUT, 'glut%sFunc'%(typeName) )
        except AttributeError as err:
            def failFunction( *args, **named ):
                from OpenGL import error
                raise error.NullFunctionError(
                    """Undefined GLUT callback function %s, check for bool(%s) before calling"""%(
                        typeName, 'glut%sFunc'%(typeName),
                    )
                )
            self.wrappedOperation = failFunction
        self.callbackType = FUNCTION_TYPE( None, *parameterTypes )
        self.CONTEXT_DATA_KEY = 'glut%sFunc'%(typeName, )
    argNames = ('function',)
    def __call__( self, function, *args ):
        if GLUT_GUARD_CALLBACKS and hasattr( function,'__call__' ):
            def safeCall( *args, **named ):
                """Safe calling of GUI callbacks, exits on failures"""
                try:
                    if not CurrentContextIsValid():
                        raise RuntimeError( """No valid context!""" )
                    return function( *args, **named )
                except Exception as err:
                    traceback.print_exc()
                    sys.stderr.write( """GLUT %s callback %s with %s,%s failed: returning None %s\n"""%(
                        self.typeName, function, args, named, err, 
                    ))
                    os._exit(1)
                    #return None
            finalFunction = safeCall
        else:
            finalFunction = function
        if hasattr( finalFunction,'__call__' ):
            cCallback = self.callbackType( finalFunction )
        else:
            cCallback = function
        # keep the function alive as long as the cCallback is...
        #cCallback.function = function
        contextdata.setValue( self.CONTEXT_DATA_KEY, cCallback )
        self.wrappedOperation( cCallback, *args )
        return cCallback
class GLUTTimerCallback( GLUTCallback ):
    """GLUT timer callbacks (completely nonstandard wrt other GLUT callbacks)"""
    def __call__( self, milliseconds, function, value ):
        cCallback = self.callbackType( function )
        # timers should de-register as soon as they are called...
        # Note: there's no good key to use! we want to allow for
        # multiple instances of the same function with the same value 
        # which means we have nothing that can store it properly...
        callbacks = contextdata.getValue( self.CONTEXT_DATA_KEY )
        if callbacks is None:
            callbacks = []
            contextdata.setValue( self.CONTEXT_DATA_KEY, callbacks )
        def deregister( value ):
            try:
                function( value )
            finally:
                for item in callbacks:
                    if item.function is deregister:
                        callbacks.remove( item )
                        item.function = None
                        break
                if not callbacks:
                    contextdata.delValue( self.CONTEXT_DATA_KEY )
        cCallback = self.callbackType( deregister )
        cCallback.function = deregister
        callbacks.append( cCallback )
        self.wrappedOperation( milliseconds, cCallback, value )
        return cCallback

class GLUTMenuCallback( object ):
    """Place to collect the GLUT Menu manipulation special code"""
    callbackType = FUNCTION_TYPE( ctypes.c_int, ctypes.c_int )
    def glutCreateMenu( cls, func ):
        """Create a new (current) menu, return small integer identifier
        
        func( int ) -- Function taking a single integer reflecting
            the user's choice, the value passed to glutAddMenuEntry
        
        return menuID (small integer)
        """
        cCallback = cls.callbackType( func )
        menu = _simple.glutCreateMenu( cCallback )
        contextdata.setValue( ('menucallback',menu), (cCallback,func) )
        return menu
    glutCreateMenu.argNames = [ 'func' ]
    glutCreateMenu = classmethod( glutCreateMenu )
    def glutDestroyMenu( cls, menu ):
        """Destroy (cleanup) the given menu
        
        Deregister's the interal pointer to the menu callback 
        
        returns None
        """
        result = _simple.glutDestroyMenu( menu )
        contextdata.delValue( ('menucallback',menu) )
        return result
    glutDestroyMenu.argNames = [ 'menu' ]
    glutDestroyMenu = classmethod( glutDestroyMenu )

glutCreateMenu = GLUTMenuCallback.glutCreateMenu
#glutCreateMenu.wrappedOperation = _simple.glutCreateMenu
glutDestroyMenu = GLUTMenuCallback.glutDestroyMenu
#glutDestroyMenu.wrappedOperation = _simple.glutDestroyMenu

glutButtonBoxFunc = GLUTCallback(
    'ButtonBox', (ctypes.c_int,ctypes.c_int), ('button','state'),
)
glutDialsFunc = GLUTCallback(
    'Dials', (ctypes.c_int,ctypes.c_int), ('dial','value'),
)
glutDisplayFunc = GLUTCallback(
    'Display', (), (),
)
glutEntryFunc = GLUTCallback(
    'Entry', (ctypes.c_int,), ('state',),
)
glutIdleFunc = GLUTCallback(
    'Idle', (), (),
)
glutJoystickFunc = GLUTCallback(
    'Joystick', (ctypes.c_uint,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('buttonMask','x','y','z'),
)
glutKeyboardFunc = GLUTCallback(
    'Keyboard', (ctypes.c_char,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutKeyboardUpFunc = GLUTCallback(
    'KeyboardUp', (ctypes.c_char,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutMenuStatusFunc = GLUTCallback(
    'MenuStatus', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('status','x','y'),
)
glutMenuStateFunc = GLUTCallback(
    'MenuState', (ctypes.c_int,), ('status',),
)
glutMotionFunc = GLUTCallback(
    'Motion', (ctypes.c_int,ctypes.c_int), ('x','y'),
)
glutMouseFunc = GLUTCallback(
    'Mouse', (ctypes.c_int,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('button','state','x','y'),
)
glutOverlayDisplayFunc = GLUTCallback(
    'OverlayDisplay', (), (),
)
glutPassiveMotionFunc = GLUTCallback(
    'PassiveMotion', (ctypes.c_int,ctypes.c_int), ('x','y'),
)
glutReshapeFunc = GLUTCallback(
    'Reshape', (ctypes.c_int,ctypes.c_int), ('width','height'),
)
glutSpaceballButtonFunc = GLUTCallback(
    'SpaceballButton', (ctypes.c_int,ctypes.c_int), ('button','state'),
)
glutSpaceballMotionFunc = GLUTCallback(
    'SpaceballMotion', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('x','y','z'),
)
glutSpaceballRotateFunc = GLUTCallback(
    'SpaceballRotate', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('x','y','z'),
)
glutSpecialFunc = GLUTCallback(
    'Special', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutSpecialUpFunc = GLUTCallback(
    'SpecialUp', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutTabletButtonFunc = GLUTCallback(
    'TabletButton', (ctypes.c_int,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('button','state','x','y',),
)
glutTabletButtonFunc = GLUTCallback(
    'TabletButton', (ctypes.c_int,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('button','state','x','y',),
)
glutTabletMotionFunc = GLUTCallback(
    'TabletMotion', (ctypes.c_int,ctypes.c_int), ('x','y',),
)
glutVisibilityFunc = GLUTCallback(
    'Visibility', (ctypes.c_int,), ('state',),
)
glutWindowStatusFunc = GLUTCallback(
    'WindowStatus', (ctypes.c_int,), ('state',),
)

# glutTimerFunc is unlike any other GLUT callback-registration...
glutTimerFunc = GLUTTimerCallback(
    'Timer', (ctypes.c_int,), ('value',),
)

INITIALIZED = False
def glutInit( *args ):
    """Initialise the GLUT library"""
    global INITIALIZED
    if INITIALIZED:
        return args
    INITIALIZED = True
    if args:
        arg,args = args[0],args[1:]
        count = None
        if isinstance(arg, integer_types):
            # raw API style, (count, values)
            count = arg
            if count != len(args):
                raise ValueError( """Specified count of %s does not match length (%s) of argument list %s"""%(
                    count, len(args), args,
                ))
        elif isinstance( arg, (bytes,unicode)):
            # passing in a sequence of strings as individual arguments
            args = (arg,)+args 
            count = len(args)
        else:
            args = arg 
            count = len(args)
    else:
        count=0
        args = []
    args = [as_8_bit(x) for x in args]
    if not count:
        count, args = 1, [as_8_bit('foo')]
    holder = (ctypes.c_char_p * len(args))()
    for i,arg in enumerate(args):
        holder[i] = arg
    count = ctypes.c_int( count )
    import os 
    currentDirectory = os.getcwd()
    try:
        # XXX need to check for error condition here...
        _base_glutInit( ctypes.byref(count), holder )
    finally:
        os.chdir( currentDirectory )
    return [
        holder[i] for i in range( count.value )
    ]
glutInit.wrappedOperation = _simple.glutInit

def glutDestroyWindow( window ):
    """Want to destroy the window, we need to do some cleanup..."""
    context = 0
    try:
        GLUT.glutSetWindow(window)
        context = contextdata.getContext()
        result = contextdata.cleanupContext( context )
        _log.info( """Cleaning up context data for window %s: %s""", window, result )
    except Exception as err:
        _log.error( """Error attempting to clean up context data for GLUT window %s: %s""", window, result )
    return _base_glutDestroyWindow( window )
glutDestroyWindow.wrappedOperation = _simple.glutDestroyWindow



'''OpenGL extension VERSION.GL_3_1

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_3_1 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_3_1.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.VERSION.GL_3_1 import *
from OpenGL.raw.GL.VERSION.GL_3_1 import _EXTENSION_NAME

def glInitGl31VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glDrawElementsInstanced.indices size not checked against 'count,type'
glDrawElementsInstanced=wrapper.wrapper(glDrawElementsInstanced).setInputArraySize(
    'indices', None
)
# INPUT glGetUniformIndices.uniformNames size not checked against 'uniformCount'
glGetUniformIndices=wrapper.wrapper(glGetUniformIndices).setOutput(
    'uniformIndices',size=_glgets._glget_size_mapping,pnameArg='uniformCount',orPassIn=True
).setInputArraySize(
    'uniformNames', None
)
# OUTPUT glGetActiveUniformsiv.params COMPSIZE(uniformCount, pname) 
# INPUT glGetActiveUniformsiv.uniformIndices size not checked against uniformCount
glGetActiveUniformsiv=wrapper.wrapper(glGetActiveUniformsiv).setInputArraySize(
    'uniformIndices', None
)
glGetActiveUniformName=wrapper.wrapper(glGetActiveUniformName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'uniformName',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
# INPUT glGetUniformBlockIndex.uniformBlockName size not checked against ''
glGetUniformBlockIndex=wrapper.wrapper(glGetUniformBlockIndex).setInputArraySize(
    'uniformBlockName', None
)
# OUTPUT glGetActiveUniformBlockiv.params COMPSIZE(program, uniformBlockIndex, pname) 
glGetActiveUniformBlockName=wrapper.wrapper(glGetActiveUniformBlockName).setOutput(
    'length',size=(1,),orPassIn=True
).setOutput(
    'uniformBlockName',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
### END AUTOGENERATED SECTION



'''OpenGL extension ARB.texture_view

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.texture_view to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows a texture's data store to be "viewed" in multiple 
	ways, either reinterpreting the data format/type as a different format/
	type with the same element size, or by clamping the mipmap level range 
	or array slice range. 
	
	The goals of this extension are to avoid having these alternate views 
	become shared mutable containers of shared mutable objects, and to add 
	the views to the API in a minimally invasive way.
	
	No new object types are added. Conceptually, a texture object is split
	into the following parts:
	
	    - A data store holding texel data.
	    - State describing which portions of the data store to use, and how
	      to interpret the data elements.
	    - An embedded sampler object.
	    - Various other texture parameters.
	
	With this extension, multiple textures can share a data store and have 
	different state describing which portions of the data store to use and how
	to interpret the data elements. The data store is refcounted and not 
	destroyed until the last texture sharing it is deleted.
	
	This extension leverages the ARB_texture_storage concept of an "immutable 
	texture". Views can only be created of textures created with TexStorage.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/texture_view.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.texture_view import *
from OpenGL.raw.GL.ARB.texture_view import _EXTENSION_NAME

def glInitTextureViewARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension NV.half_float

This module customises the behaviour of the 
OpenGL.raw.GL.NV.half_float to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces a new storage format and data type for
	half-precision (16-bit) floating-point quantities.  The floating-point
	format is very similar to the IEEE single-precision floating-point
	standard, except that it has only 5 exponent bits and 10 mantissa bits.
	Half-precision floats are smaller than full precision floats and provide a
	larger dynamic range than similarly-sized normalized scalar data types.
	
	This extension allows applications to use half-precision floating point
	data when specifying vertices or pixel data.  It adds new commands to
	specify vertex attributes using the new data type, and extends the
	existing vertex array and image specification commands to accept the new
	data type.
	
	This storage format is also used to represent 16-bit components in the
	floating-point frame buffers, as defined in the NV_float_buffer extension.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/half_float.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.half_float import *
from OpenGL.raw.GL.NV.half_float import _EXTENSION_NAME

def glInitHalfFloatNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glVertex2hvNV=wrapper.wrapper(glVertex2hvNV).setInputArraySize(
    'v', 2
)
glVertex3hvNV=wrapper.wrapper(glVertex3hvNV).setInputArraySize(
    'v', 3
)
glVertex4hvNV=wrapper.wrapper(glVertex4hvNV).setInputArraySize(
    'v', 4
)
glNormal3hvNV=wrapper.wrapper(glNormal3hvNV).setInputArraySize(
    'v', 3
)
glColor3hvNV=wrapper.wrapper(glColor3hvNV).setInputArraySize(
    'v', 3
)
glColor4hvNV=wrapper.wrapper(glColor4hvNV).setInputArraySize(
    'v', 4
)
glTexCoord1hvNV=wrapper.wrapper(glTexCoord1hvNV).setInputArraySize(
    'v', 1
)
glTexCoord2hvNV=wrapper.wrapper(glTexCoord2hvNV).setInputArraySize(
    'v', 2
)
glTexCoord3hvNV=wrapper.wrapper(glTexCoord3hvNV).setInputArraySize(
    'v', 3
)
glTexCoord4hvNV=wrapper.wrapper(glTexCoord4hvNV).setInputArraySize(
    'v', 4
)
glMultiTexCoord1hvNV=wrapper.wrapper(glMultiTexCoord1hvNV).setInputArraySize(
    'v', 1
)
glMultiTexCoord2hvNV=wrapper.wrapper(glMultiTexCoord2hvNV).setInputArraySize(
    'v', 2
)
glMultiTexCoord3hvNV=wrapper.wrapper(glMultiTexCoord3hvNV).setInputArraySize(
    'v', 3
)
glMultiTexCoord4hvNV=wrapper.wrapper(glMultiTexCoord4hvNV).setInputArraySize(
    'v', 4
)
glFogCoordhvNV=wrapper.wrapper(glFogCoordhvNV).setInputArraySize(
    'fog', 1
)
glSecondaryColor3hvNV=wrapper.wrapper(glSecondaryColor3hvNV).setInputArraySize(
    'v', 3
)
glVertexWeighthvNV=wrapper.wrapper(glVertexWeighthvNV).setInputArraySize(
    'weight', 1
)
glVertexAttrib1hvNV=wrapper.wrapper(glVertexAttrib1hvNV).setInputArraySize(
    'v', 1
)
glVertexAttrib2hvNV=wrapper.wrapper(glVertexAttrib2hvNV).setInputArraySize(
    'v', 2
)
glVertexAttrib3hvNV=wrapper.wrapper(glVertexAttrib3hvNV).setInputArraySize(
    'v', 3
)
glVertexAttrib4hvNV=wrapper.wrapper(glVertexAttrib4hvNV).setInputArraySize(
    'v', 4
)
# INPUT glVertexAttribs1hvNV.v size not checked against n
glVertexAttribs1hvNV=wrapper.wrapper(glVertexAttribs1hvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs2hvNV.v size not checked against n
glVertexAttribs2hvNV=wrapper.wrapper(glVertexAttribs2hvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs3hvNV.v size not checked against n
glVertexAttribs3hvNV=wrapper.wrapper(glVertexAttribs3hvNV).setInputArraySize(
    'v', None
)
# INPUT glVertexAttribs4hvNV.v size not checked against n
glVertexAttribs4hvNV=wrapper.wrapper(glVertexAttribs4hvNV).setInputArraySize(
    'v', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_debug_output'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_debug_output',error_checker=_errors._error_checker)
GL_DEBUG_CALLBACK_FUNCTION_ARB=_C('GL_DEBUG_CALLBACK_FUNCTION_ARB',0x8244)
GL_DEBUG_CALLBACK_USER_PARAM_ARB=_C('GL_DEBUG_CALLBACK_USER_PARAM_ARB',0x8245)
GL_DEBUG_LOGGED_MESSAGES_ARB=_C('GL_DEBUG_LOGGED_MESSAGES_ARB',0x9145)
GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH_ARB=_C('GL_DEBUG_NEXT_LOGGED_MESSAGE_LENGTH_ARB',0x8243)
GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB=_C('GL_DEBUG_OUTPUT_SYNCHRONOUS_ARB',0x8242)
GL_DEBUG_SEVERITY_HIGH_ARB=_C('GL_DEBUG_SEVERITY_HIGH_ARB',0x9146)
GL_DEBUG_SEVERITY_LOW_ARB=_C('GL_DEBUG_SEVERITY_LOW_ARB',0x9148)
GL_DEBUG_SEVERITY_MEDIUM_ARB=_C('GL_DEBUG_SEVERITY_MEDIUM_ARB',0x9147)
GL_DEBUG_SOURCE_API_ARB=_C('GL_DEBUG_SOURCE_API_ARB',0x8246)
GL_DEBUG_SOURCE_APPLICATION_ARB=_C('GL_DEBUG_SOURCE_APPLICATION_ARB',0x824A)
GL_DEBUG_SOURCE_OTHER_ARB=_C('GL_DEBUG_SOURCE_OTHER_ARB',0x824B)
GL_DEBUG_SOURCE_SHADER_COMPILER_ARB=_C('GL_DEBUG_SOURCE_SHADER_COMPILER_ARB',0x8248)
GL_DEBUG_SOURCE_THIRD_PARTY_ARB=_C('GL_DEBUG_SOURCE_THIRD_PARTY_ARB',0x8249)
GL_DEBUG_SOURCE_WINDOW_SYSTEM_ARB=_C('GL_DEBUG_SOURCE_WINDOW_SYSTEM_ARB',0x8247)
GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR_ARB=_C('GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR_ARB',0x824D)
GL_DEBUG_TYPE_ERROR_ARB=_C('GL_DEBUG_TYPE_ERROR_ARB',0x824C)
GL_DEBUG_TYPE_OTHER_ARB=_C('GL_DEBUG_TYPE_OTHER_ARB',0x8251)
GL_DEBUG_TYPE_PERFORMANCE_ARB=_C('GL_DEBUG_TYPE_PERFORMANCE_ARB',0x8250)
GL_DEBUG_TYPE_PORTABILITY_ARB=_C('GL_DEBUG_TYPE_PORTABILITY_ARB',0x824F)
GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR_ARB=_C('GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR_ARB',0x824E)
GL_MAX_DEBUG_LOGGED_MESSAGES_ARB=_C('GL_MAX_DEBUG_LOGGED_MESSAGES_ARB',0x9144)
GL_MAX_DEBUG_MESSAGE_LENGTH_ARB=_C('GL_MAX_DEBUG_MESSAGE_LENGTH_ARB',0x9143)
@_f
@_p.types(None,_cs.GLDEBUGPROCARB,ctypes.c_void_p)
def glDebugMessageCallbackARB(callback,userParam):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray,_cs.GLboolean)
def glDebugMessageControlARB(source,type,severity,count,ids,enabled):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLuint,_cs.GLenum,_cs.GLsizei,arrays.GLcharArray)
def glDebugMessageInsertARB(source,type,id,severity,length,buf):pass
@_f
@_p.types(_cs.GLuint,_cs.GLuint,_cs.GLsizei,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLuintArray,arrays.GLsizeiArray,arrays.GLcharArray)
def glGetDebugMessageLogARB(count,bufSize,sources,types,ids,severities,lengths,messageLog):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_transform_feedback2'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_transform_feedback2',error_checker=_errors._error_checker)
GL_TRANSFORM_FEEDBACK=_C('GL_TRANSFORM_FEEDBACK',0x8E22)
GL_TRANSFORM_FEEDBACK_BINDING=_C('GL_TRANSFORM_FEEDBACK_BINDING',0x8E25)
GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE=_C('GL_TRANSFORM_FEEDBACK_BUFFER_ACTIVE',0x8E24)
GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED=_C('GL_TRANSFORM_FEEDBACK_BUFFER_PAUSED',0x8E23)
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glBindTransformFeedback(target,id):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glDeleteTransformFeedbacks(n,ids):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLuint)
def glDrawTransformFeedback(mode,id):pass
@_f
@_p.types(None,_cs.GLsizei,arrays.GLuintArray)
def glGenTransformFeedbacks(n,ids):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint)
def glIsTransformFeedback(id):pass
@_f
@_p.types(None,)
def glPauseTransformFeedback():pass
@_f
@_p.types(None,)
def glResumeTransformFeedback():pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_cmyka'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_cmyka',error_checker=_errors._error_checker)
GL_CMYKA_EXT=_C('GL_CMYKA_EXT',0x800D)
GL_CMYK_EXT=_C('GL_CMYK_EXT',0x800C)
GL_PACK_CMYK_HINT_EXT=_C('GL_PACK_CMYK_HINT_EXT',0x800E)
GL_UNPACK_CMYK_HINT_EXT=_C('GL_UNPACK_CMYK_HINT_EXT',0x800F)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_pixel_transform'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_pixel_transform',error_checker=_errors._error_checker)
GL_AVERAGE_EXT=_C('GL_AVERAGE_EXT',0x8335)
GL_CUBIC_EXT=_C('GL_CUBIC_EXT',0x8334)
GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT=_C('GL_MAX_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',0x8337)
GL_PIXEL_CUBIC_WEIGHT_EXT=_C('GL_PIXEL_CUBIC_WEIGHT_EXT',0x8333)
GL_PIXEL_MAG_FILTER_EXT=_C('GL_PIXEL_MAG_FILTER_EXT',0x8331)
GL_PIXEL_MIN_FILTER_EXT=_C('GL_PIXEL_MIN_FILTER_EXT',0x8332)
GL_PIXEL_TRANSFORM_2D_EXT=_C('GL_PIXEL_TRANSFORM_2D_EXT',0x8330)
GL_PIXEL_TRANSFORM_2D_MATRIX_EXT=_C('GL_PIXEL_TRANSFORM_2D_MATRIX_EXT',0x8338)
GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT=_C('GL_PIXEL_TRANSFORM_2D_STACK_DEPTH_EXT',0x8336)
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glGetPixelTransformParameterfvEXT(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glGetPixelTransformParameterivEXT(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLfloat)
def glPixelTransformParameterfEXT(target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLfloatArray)
def glPixelTransformParameterfvEXT(target,pname,params):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,_cs.GLint)
def glPixelTransformParameteriEXT(target,pname,param):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLenum,arrays.GLintArray)
def glPixelTransformParameterivEXT(target,pname,params):pass



'''OpenGL extension EXT.separate_shader_objects

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.separate_shader_objects to provide a more 
Python-friendly API

Overview (from the spec)
	
	Prior to this extension, GLSL requires multiple shader domains
	(vertex, fragment, geometry) to be linked into a single monolithic
	program object to specify a GLSL shader for each domain.
	
	While GLSL's monolithic approach has some advantages for
	optimizing shaders as a unit that span multiple domains, all
	existing GPU hardware supports the more flexible mix-and-match
	approach.
	
	HLSL9, Cg, the prior OpenGL assembly program extensions, and game
	console programmers favor a more flexible "mix-and-match" approach to
	specifying shaders independently for these different shader domains.
	Many developers build their shader content around the mix-and-match
	approach where they can use a single vertex shader with multiple
	fragment shaders (or vice versa).
	
	This keep-it-simple extension adapts the "mix-and-match" shader
	domain model for GLSL so different GLSL program objects can be bound
	to different shader domains.
	
	This extension redefines the operation of glUseProgram(GLenum program)
	to be equivalent to:
	
	    glUseShaderProgramEXT(GL_VERTEX_SHADER, program);
	    glUseShaderProgramEXT(GL_GEOMETRY_SHADER_EXT, program);
	    glUseShaderProgramEXT(GL_FRAGMENT_SHADER, program);
	    glActiveProgramEXT(program);
	
	You can also call these commands separately to bind each respective
	domain.  The GL_VERTEX_SHADER, GL_GEOMETRY_SHADER_EXT, and
	GL_FRAGMENT_SHADER tokens refer to the conventional vertex, geometry,
	and fragment domains respectively.  glActiveProgramEXT specifies
	the program that glUniform* commands will update.
	
	Separate linking creates the possibility that certain output varyings
	of a shader may go unread by the subsequent shader inputting varyings.
	In this case, the output varyings are simply ignored.  It is also
	possible input varyings from a shader may not be written as output
	varyings of a preceding shader.  In this case, the unwritten input
	varying values are undefined.  Implementations are encouraged to
	zero these undefined input varying values.
	
	This extension is a proof-of-concept that separate shader objects
	can work for GLSL and a response to repeated requests for this
	functionality.  There are various loose ends, particularly when
	dealing with user-defined varyings.  The hope is a future extension
	will improve this situation.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/separate_shader_objects.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.separate_shader_objects import *
from OpenGL.raw.GLES2.EXT.separate_shader_objects import _EXTENSION_NAME

def glInitSeparateShaderObjectsEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glCreateShaderProgramvEXT.strings size not checked against count
glCreateShaderProgramvEXT=wrapper.wrapper(glCreateShaderProgramvEXT).setInputArraySize(
    'strings', None
)
# INPUT glDeleteProgramPipelinesEXT.pipelines size not checked against n
glDeleteProgramPipelinesEXT=wrapper.wrapper(glDeleteProgramPipelinesEXT).setInputArraySize(
    'pipelines', None
)
# INPUT glGenProgramPipelinesEXT.pipelines size not checked against n
glGenProgramPipelinesEXT=wrapper.wrapper(glGenProgramPipelinesEXT).setInputArraySize(
    'pipelines', None
)
# INPUT glGetProgramPipelineInfoLogEXT.infoLog size not checked against bufSize
glGetProgramPipelineInfoLogEXT=wrapper.wrapper(glGetProgramPipelineInfoLogEXT).setInputArraySize(
    'infoLog', None
).setInputArraySize(
    'length', 1
)
# INPUT glProgramUniform1fvEXT.value size not checked against count
glProgramUniform1fvEXT=wrapper.wrapper(glProgramUniform1fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1ivEXT.value size not checked against count
glProgramUniform1ivEXT=wrapper.wrapper(glProgramUniform1ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2fvEXT.value size not checked against count*2
glProgramUniform2fvEXT=wrapper.wrapper(glProgramUniform2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2ivEXT.value size not checked against count*2
glProgramUniform2ivEXT=wrapper.wrapper(glProgramUniform2ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3fvEXT.value size not checked against count*3
glProgramUniform3fvEXT=wrapper.wrapper(glProgramUniform3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3ivEXT.value size not checked against count*3
glProgramUniform3ivEXT=wrapper.wrapper(glProgramUniform3ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4fvEXT.value size not checked against count*4
glProgramUniform4fvEXT=wrapper.wrapper(glProgramUniform4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4ivEXT.value size not checked against count*4
glProgramUniform4ivEXT=wrapper.wrapper(glProgramUniform4ivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2fvEXT.value size not checked against count*4
glProgramUniformMatrix2fvEXT=wrapper.wrapper(glProgramUniformMatrix2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3fvEXT.value size not checked against count*9
glProgramUniformMatrix3fvEXT=wrapper.wrapper(glProgramUniformMatrix3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4fvEXT.value size not checked against count*16
glProgramUniformMatrix4fvEXT=wrapper.wrapper(glProgramUniformMatrix4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform1uivEXT.value size not checked against count
glProgramUniform1uivEXT=wrapper.wrapper(glProgramUniform1uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform2uivEXT.value size not checked against count*2
glProgramUniform2uivEXT=wrapper.wrapper(glProgramUniform2uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform3uivEXT.value size not checked against count*3
glProgramUniform3uivEXT=wrapper.wrapper(glProgramUniform3uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniform4uivEXT.value size not checked against count*4
glProgramUniform4uivEXT=wrapper.wrapper(glProgramUniform4uivEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4fvEXT.value size not checked against count*16
glProgramUniformMatrix4fvEXT=wrapper.wrapper(glProgramUniformMatrix4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x3fvEXT.value size not checked against count*6
glProgramUniformMatrix2x3fvEXT=wrapper.wrapper(glProgramUniformMatrix2x3fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x2fvEXT.value size not checked against count*6
glProgramUniformMatrix3x2fvEXT=wrapper.wrapper(glProgramUniformMatrix3x2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix2x4fvEXT.value size not checked against count*8
glProgramUniformMatrix2x4fvEXT=wrapper.wrapper(glProgramUniformMatrix2x4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x2fvEXT.value size not checked against count*8
glProgramUniformMatrix4x2fvEXT=wrapper.wrapper(glProgramUniformMatrix4x2fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix3x4fvEXT.value size not checked against count*12
glProgramUniformMatrix3x4fvEXT=wrapper.wrapper(glProgramUniformMatrix3x4fvEXT).setInputArraySize(
    'value', None
)
# INPUT glProgramUniformMatrix4x3fvEXT.value size not checked against count*12
glProgramUniformMatrix4x3fvEXT=wrapper.wrapper(glProgramUniformMatrix4x3fvEXT).setInputArraySize(
    'value', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_OES_texture_storage_multisample_2d_array'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_OES_texture_storage_multisample_2d_array',error_checker=_errors._error_checker)
GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY_OES=_C('GL_INT_SAMPLER_2D_MULTISAMPLE_ARRAY_OES',0x910C)
GL_SAMPLER_2D_MULTISAMPLE_ARRAY_OES=_C('GL_SAMPLER_2D_MULTISAMPLE_ARRAY_OES',0x910B)
GL_TEXTURE_2D_MULTISAMPLE_ARRAY_OES=_C('GL_TEXTURE_2D_MULTISAMPLE_ARRAY_OES',0x9102)
GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY_OES=_C('GL_TEXTURE_BINDING_2D_MULTISAMPLE_ARRAY_OES',0x9105)
GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY_OES=_C('GL_UNSIGNED_INT_SAMPLER_2D_MULTISAMPLE_ARRAY_OES',0x910D)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLboolean)
def glTexStorage3DMultisampleOES(target,samples,internalformat,width,height,depth,fixedsamplelocations):pass



'''OpenGL extension ARB.framebuffer_sRGB

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.framebuffer_sRGB to provide a more 
Python-friendly API

Overview (from the spec)
	
	Conventionally, OpenGL assumes framebuffer color components are stored
	in a linear color space.  In particular, framebuffer blending is a
	linear operation.
	
	The sRGB color space is based on typical (non-linear) monitor
	characteristics expected in a dimly lit office.  It has been
	standardized by the International Electrotechnical Commission (IEC)
	as IEC 61966-2-1. The sRGB color space roughly corresponds to 2.2
	gamma correction.
	
	This extension adds a framebuffer capability for sRGB framebuffer
	update and blending.  When blending is disabled but the new sRGB
	updated mode is enabled (assume the framebuffer supports the
	capability), high-precision linear color component values for red,
	green, and blue generated by fragment coloring are encoded for sRGB
	prior to being written into the framebuffer.  When blending is enabled
	along with the new sRGB update mode, red, green, and blue framebuffer
	color components are treated as sRGB values that are converted to
	linear color values, blended with the high-precision color values
	generated by fragment coloring, and then the blend result is encoded
	for sRGB just prior to being written into the framebuffer.
	
	The primary motivation for this extension is that it allows OpenGL
	applications to render into a framebuffer that is scanned to a monitor
	configured to assume framebuffer color values are sRGB encoded.
	This assumption is roughly true of most PC monitors with default
	gamma correction.  This allows applications to achieve faithful
	color reproduction for OpenGL rendering without adjusting the
	monitor's gamma correction.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/framebuffer_sRGB.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.framebuffer_sRGB import *
from OpenGL.raw.GL.ARB.framebuffer_sRGB import _EXTENSION_NAME

def glInitFramebufferSrgbARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.EGL import _types as _cs
# End users want this...
from OpenGL.raw.EGL._types import *
from OpenGL.raw.EGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'EGL_EXT_image_dma_buf_import'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.EGL,'EGL_EXT_image_dma_buf_import',error_checker=_errors._error_checker)
EGL_DMA_BUF_PLANE0_FD_EXT=_C('EGL_DMA_BUF_PLANE0_FD_EXT',0x3272)
EGL_DMA_BUF_PLANE0_OFFSET_EXT=_C('EGL_DMA_BUF_PLANE0_OFFSET_EXT',0x3273)
EGL_DMA_BUF_PLANE0_PITCH_EXT=_C('EGL_DMA_BUF_PLANE0_PITCH_EXT',0x3274)
EGL_DMA_BUF_PLANE1_FD_EXT=_C('EGL_DMA_BUF_PLANE1_FD_EXT',0x3275)
EGL_DMA_BUF_PLANE1_OFFSET_EXT=_C('EGL_DMA_BUF_PLANE1_OFFSET_EXT',0x3276)
EGL_DMA_BUF_PLANE1_PITCH_EXT=_C('EGL_DMA_BUF_PLANE1_PITCH_EXT',0x3277)
EGL_DMA_BUF_PLANE2_FD_EXT=_C('EGL_DMA_BUF_PLANE2_FD_EXT',0x3278)
EGL_DMA_BUF_PLANE2_OFFSET_EXT=_C('EGL_DMA_BUF_PLANE2_OFFSET_EXT',0x3279)
EGL_DMA_BUF_PLANE2_PITCH_EXT=_C('EGL_DMA_BUF_PLANE2_PITCH_EXT',0x327A)
EGL_ITU_REC2020_EXT=_C('EGL_ITU_REC2020_EXT',0x3281)
EGL_ITU_REC601_EXT=_C('EGL_ITU_REC601_EXT',0x327F)
EGL_ITU_REC709_EXT=_C('EGL_ITU_REC709_EXT',0x3280)
EGL_LINUX_DMA_BUF_EXT=_C('EGL_LINUX_DMA_BUF_EXT',0x3270)
EGL_LINUX_DRM_FOURCC_EXT=_C('EGL_LINUX_DRM_FOURCC_EXT',0x3271)
EGL_SAMPLE_RANGE_HINT_EXT=_C('EGL_SAMPLE_RANGE_HINT_EXT',0x327C)
EGL_YUV_CHROMA_HORIZONTAL_SITING_HINT_EXT=_C('EGL_YUV_CHROMA_HORIZONTAL_SITING_HINT_EXT',0x327D)
EGL_YUV_CHROMA_SITING_0_5_EXT=_C('EGL_YUV_CHROMA_SITING_0_5_EXT',0x3285)
EGL_YUV_CHROMA_SITING_0_EXT=_C('EGL_YUV_CHROMA_SITING_0_EXT',0x3284)
EGL_YUV_CHROMA_VERTICAL_SITING_HINT_EXT=_C('EGL_YUV_CHROMA_VERTICAL_SITING_HINT_EXT',0x327E)
EGL_YUV_COLOR_SPACE_HINT_EXT=_C('EGL_YUV_COLOR_SPACE_HINT_EXT',0x327B)
EGL_YUV_FULL_RANGE_EXT=_C('EGL_YUV_FULL_RANGE_EXT',0x3282)
EGL_YUV_NARROW_RANGE_EXT=_C('EGL_YUV_NARROW_RANGE_EXT',0x3283)




'''OpenGL extension ARB.sample_shading

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.sample_shading to provide a more 
Python-friendly API

Overview (from the spec)
	
	In standard multisample rendering, an implementation is allowed to
	assign the same color and texture coordinate values to each sample,
	which then allows the optimization where the shader is only
	evaluated once and then distributed to the samples that have been
	determined to be covered by the primitive currently being
	rasterized. This can cause aliasing where the input color and
	texture coordinates are used to generate a result that doesn't
	antialias itself, for example with alpha-tested transparency.
	
	This extension adds the ability to explicitly request that an
	implementation use a minimum number of unique set of fragment
	computation inputs when multisampling a pixel. Specifying such a
	requirement can reduce aliasing that results from evaluating the
	fragment computations too few times per pixel.
	
	This extension adds new global state that controls the minimum
	number of samples for which attribute data is independently
	interpolated. When enabled, all operations that were traditionally
	executed per-fragment operate independently on each sample.
	
	This also extends the shading language to allow control over the
	sample being processed. This includes built-in fragment input
	variables identifying the sample number and position being processed
	when executing fragment shaders per sample.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/sample_shading.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.sample_shading import *
from OpenGL.raw.GL.ARB.sample_shading import _EXTENSION_NAME

def glInitSampleShadingARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_ARB_texture_view'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_ARB_texture_view',error_checker=_errors._error_checker)
GL_TEXTURE_IMMUTABLE_LEVELS=_C('GL_TEXTURE_IMMUTABLE_LEVELS',0x82DF)
GL_TEXTURE_VIEW_MIN_LAYER=_C('GL_TEXTURE_VIEW_MIN_LAYER',0x82DD)
GL_TEXTURE_VIEW_MIN_LEVEL=_C('GL_TEXTURE_VIEW_MIN_LEVEL',0x82DB)
GL_TEXTURE_VIEW_NUM_LAYERS=_C('GL_TEXTURE_VIEW_NUM_LAYERS',0x82DE)
GL_TEXTURE_VIEW_NUM_LEVELS=_C('GL_TEXTURE_VIEW_NUM_LEVELS',0x82DC)
@_f
@_p.types(None,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLenum,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glTextureView(texture,target,origtexture,internalformat,minlevel,numlevels,minlayer,numlayers):pass



'''OpenGL extension ARB.robustness

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.robustness to provide a more 
Python-friendly API

Overview (from the spec)
	
	Several recent trends in how OpenGL integrates into modern computer
	systems have created new requirements for robustness and security
	for OpenGL rendering contexts.
	
	Additionally GPU architectures now support hardware fault detection;
	for example, video memory supporting ECC (error correcting codes)
	and error detection.  OpenGL contexts should be capable of recovering
	from hardware faults such as uncorrectable memory errors.  Along with
	recovery from such hardware faults, the recovery mechanism can
	also allow recovery from video memory access exceptions and system
	software failures.  System software failures can be due to device
	changes or driver failures.
	
	Demands for increased software robustness and concerns about malware
	exploiting buffer overflows have lead API designers to provide
	additional "safe" APIs that bound the amount of data returned by
	an API query.  For example, the safer "snprintf" or "_snprintf"
	routines are prefered over "sprintf".
	
	The OpenGL API has many such robustness perils.  OpenGL queries
	return (write) some number of bytes to a buffer indicated by a
	pointer parameter.  The exact number of bytes written by existing
	OpenGL queries is not expressed directly by any specific parameter;
	instead the number of bytes returned is a complex function of one
	or more query arguments, sometimes context state such as pixel
	store modes or the active texture selector, and the current state
	of an object (such as a texture level's number of total texels).
	By the standards of modern API design, such queries are not "safe".
	Making these queries safer involves introducing a new query API with
	an additional parameter that specifies the number of bytes in the
	buffer and never writing bytes beyond that limit.
	
	Multi-threaded use of OpenGL contexts in a "share group" allow
	sharing of objects such as textures and programs.  Such sharing in
	conjunction with concurrent OpenGL commands stream execution by two
	or more contexts introduces hazards whereby one context can change
	objects in ways that can cause buffer overflows for another context's
	OpenGL queries.
	
	The original ARB_vertex_buffer_object extension includes an issue
	that explicitly states program termination is allowed when
	out-of-bounds vertex buffer object fetches occur. Modern GPUs
	capable of DirectX 10 enforce the well-defined behavior of always
	returning zero values for indices or non-fixed components in this
	case. Older GPUs may require extra checks to enforce well-defined
	(and termination free) behavior, but this expense is warranted when
	processing potentially untrusted content.
	
	The intent of this extension is to address some specific robustness
	goals:
	
	*   For all existing OpenGL queries, provide additional "safe" APIs 
	    that limit data written to user pointers to a buffer size in 
	    bytes that is an explicit additional parameter of the query.
	
	*   Provide a mechanism for an OpenGL application to learn about
	    graphics resets that affect the context.  When a graphics reset
	    occurs, the OpenGL context becomes unusable and the application
	    must create a new context to continue operation. Detecting a
	    graphics reset happens through an inexpensive query.
	
	*   Provide an enable to guarantee that out-of-bounds buffer object
	    accesses by the GPU will have deterministic behavior and preclude
	    application instability or termination due to an incorrect buffer
	    access.  Such accesses include vertex buffer fetches of
	    attributes and indices, and indexed reads of uniforms or
	    parameters from buffers.
	
	In one anticipated usage model, WebGL contexts may make use of these
	robust features to grant greater stability when using untrusted code.
	WebGL contexts cannot call OpenGL commands directly but rather must
	route all OpenGL API calls through the web browser.  It is then the
	web browser that configures the context, using the commands in this
	extension, to enforce safe behavior. In this scenario, the WebGL
	content cannot specify or change the use of this extension's features
	itself; the web browser enforces this policy.
	
	There are other well-known robustness issues with the OpenGL API
	which this extension does not address.  For example, selector-based
	OpenGL commands are a well-known source of programming errors.
	Code to manipulate texture state may assume the active texture
	selector is set appropriately when an intervening function call
	obscures a change to the active texture state resulting in
	incorrectly updated or queried state.  The EXT_direct_state_access
	extension introduces selector-free OpenGL commands and queries to
	address that particular issue so this extension does not.
	
	The intent of this extension is NOT to deprecate any existing API
	and thereby introduce compatibility issues and coding burdens on
	existing code, but rather to provide new APIs to ensure a level of
	robustness commensurate with the expectations of modern applications
	of OpenGL.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ARB/robustness.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.ARB.robustness import *
from OpenGL.raw.GL.ARB.robustness import _EXTENSION_NAME

def glInitRobustnessARB():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glGetnTexImageARB=wrapper.wrapper(glGetnTexImageARB).setOutput(
    'img',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glReadnPixelsARB=wrapper.wrapper(glReadnPixelsARB).setOutput(
    'data',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnCompressedTexImageARB=wrapper.wrapper(glGetnCompressedTexImageARB).setOutput(
    'img',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnUniformfvARB=wrapper.wrapper(glGetnUniformfvARB).setOutput(
    'params',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnUniformivARB=wrapper.wrapper(glGetnUniformivARB).setOutput(
    'params',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnUniformuivARB=wrapper.wrapper(glGetnUniformuivARB).setOutput(
    'params',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnUniformdvARB=wrapper.wrapper(glGetnUniformdvARB).setOutput(
    'params',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnMapdvARB=wrapper.wrapper(glGetnMapdvARB).setOutput(
    'v',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnMapfvARB=wrapper.wrapper(glGetnMapfvARB).setOutput(
    'v',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnMapivARB=wrapper.wrapper(glGetnMapivARB).setOutput(
    'v',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnPixelMapfvARB=wrapper.wrapper(glGetnPixelMapfvARB).setOutput(
    'values',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnPixelMapuivARB=wrapper.wrapper(glGetnPixelMapuivARB).setOutput(
    'values',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnPixelMapusvARB=wrapper.wrapper(glGetnPixelMapusvARB).setOutput(
    'values',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnPolygonStippleARB=wrapper.wrapper(glGetnPolygonStippleARB).setOutput(
    'pattern',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnColorTableARB=wrapper.wrapper(glGetnColorTableARB).setOutput(
    'table',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnConvolutionFilterARB=wrapper.wrapper(glGetnConvolutionFilterARB).setOutput(
    'image',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnSeparableFilterARB=wrapper.wrapper(glGetnSeparableFilterARB).setOutput(
    'column',size=lambda x:(x,),pnameArg='columnBufSize',orPassIn=True
).setOutput(
    'row',size=lambda x:(x,),pnameArg='rowBufSize',orPassIn=True
).setOutput(
    'span',size=(0,),orPassIn=True
)
glGetnHistogramARB=wrapper.wrapper(glGetnHistogramARB).setOutput(
    'values',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
glGetnMinmaxARB=wrapper.wrapper(glGetnMinmaxARB).setOutput(
    'values',size=lambda x:(x,),pnameArg='bufSize',orPassIn=True
)
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.cull_vertex

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.cull_vertex to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension introduces a method for culling vertexes in object
	space based on the value of the dot product between the normal at
	the vertex and a culling eye direction.
	
	Culling a polygon by examining its vertexes in object space can be
	more efficient than screen space polygon culling since the transformation
	to screen space (which may include a division by w) can be avoided for
	culled vertexes.  Also, vertex culling can be computed before vertexes
	are assembled into primitives.  This is a useful property when drawing
	meshes with shared vertexes, since a vertex can be culled once, and the
	resulting state can be used for all primitives which share the vertex.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/cull_vertex.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.EXT.cull_vertex import *
from OpenGL.raw.GL.EXT.cull_vertex import _EXTENSION_NAME

def glInitCullVertexEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

glCullParameterdvEXT=wrapper.wrapper(glCullParameterdvEXT).setOutput(
    'params',size=(4,),orPassIn=True
)
glCullParameterfvEXT=wrapper.wrapper(glCullParameterfvEXT).setOutput(
    'params',size=(4,),orPassIn=True
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_EXT_blend_color'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_EXT_blend_color',error_checker=_errors._error_checker)
GL_BLEND_COLOR_EXT=_C('GL_BLEND_COLOR_EXT',0x8005)
GL_CONSTANT_ALPHA_EXT=_C('GL_CONSTANT_ALPHA_EXT',0x8003)
GL_CONSTANT_COLOR_EXT=_C('GL_CONSTANT_COLOR_EXT',0x8001)
GL_ONE_MINUS_CONSTANT_ALPHA_EXT=_C('GL_ONE_MINUS_CONSTANT_ALPHA_EXT',0x8004)
GL_ONE_MINUS_CONSTANT_COLOR_EXT=_C('GL_ONE_MINUS_CONSTANT_COLOR_EXT',0x8002)
@_f
@_p.types(None,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat,_cs.GLfloat)
def glBlendColorEXT(red,green,blue,alpha):pass



'''OpenGL extension SGI.cushion

This module customises the behaviour of the 
OpenGL.raw.GLX.SGI.cushion to provide a more 
Python-friendly API

Overview (from the spec)
	
	    This extension is directed at constant frame-rate applications.  Such
	    applications are written to ensure that a new image is generated every
	    N video frame periods, where N is itself a small constant integer.
	    If the application is unable to generate a frame within N video frame
	    periods, it is said to have dropped a frame.  Dropping a frame is a
	    *bad thing*.
	
	    Constant frame-rate applications make every effort to avoid dropping
	    frames.  In particular, they monitor the utilization of graphics
	    resources during the rendering of the current frame in order to predict
	    the behavior of subsequent frames.  If such prediction indicates that
	    a frame may be dropped, the rendering complexity of the frame is
	    reduced (e.g. by using models of lower geometric resolution) so as to
	    avoid the overload condition.
	
	    Unfortunately, because exact prediction is not possible, and because
	    there is no elasticity in the buffering of images, it is necessary
	    for constant frame-rate applications to under utilize the graphics
	    hardware.  This extension adds elasticity to the buffering of completed
	    images, in order to allow constant frame-rate applications to make full
	    use of the available graphics computation without dropping frames.
	    It further allows this elasticity to be controlled by the application in
	    order to minimize the introduction of latency that could otherwise
	    occur.
	
	    Applications that will benefit from this extension include simulation,
	    walk-through, and multimedia playback.
	
	    WARNING - Silicon Graphics has filed for patent protection for some
		      of the techniques described in this extension document.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/SGI/cushion.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLX import _types, _glgets
from OpenGL.raw.GLX.SGI.cushion import *
from OpenGL.raw.GLX.SGI.cushion import _EXTENSION_NAME

def glInitCushionSGI():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION